<?php

/*

A SIMPLE PROGRAM TO COMPUTE THE RELATIVE SIZES OF THE SUN AND MOON
AT THE CURRENT MOMENT AND OVERLAY THEIR RELATIVE DISKS FOR CURRENT
DISK SIZE COMPARISONS.

SUN  = BRIGHT YELLOW DISK / DARK YELLOW DISK
MOON = BLACK / DARK YELLOW DISK

*/


/* -------------------------------------------
   Initialize output buffer for using cookies.
*/
   ob_start();

// Set to testing mode (TRUE), if desired.
   $InTestingMode = FALSE;



// Define 78-character horizontal bars. These
// can be called globally as block separators.
   $_HR1_ = Str_Repeat("&#183;", 78);
   $_HR2_ = Str_Repeat('-', 78);
   $_HR3_ = Str_Repeat('=', 78);
   $_HR4_ = Str_Repeat('*', 78);
   $_HR5_ = Str_Repeat("&#151;", 78);

   $_HR_ = array();

   for($i=1;   $i < 80;   $i++)
  {
   $chars = Str_Repeat("=", $i);
   $_HR_[$i] = $chars;
  }


// CONSTANTS
//
// ------------------------------------
// Speed of light in meters per second.
// Taken from NASA/JPL DE430 constants.

// ---------------------------------------------
// Due to the way the length of a second and the
// length of the meter are EXACTLY defined, this
// speed of light is also an EXACT definition.

   define ("CLIGHT", "299792458");

// -----------------------------------------------
// Standard Earth gravity (1 G) acceleration (SI).

   define ("G_STD", "9.80665");

// --------------------------------
// Universal gravity constant (SI).

   define ("G_UNI", "0.000000000066742");

// -------------------------------------------------------------------
// Define some EXACT standard metric length/distance interconversion
// factors in terms of meters per unit (M_PER_XXX).  This is the
// factor by which to multiply the units to obtain equivalent meters.

   define ("M_PER_A"   , "0.0000000001");  // m per angstrom
   define ("M_PER_NM"  , "0.000000001");   // m per nanometer
   define ("M_PER_UM"  , "0.000001");      // m per micrometer (micron)
   define ("M_PER_MM"  , "0.001");         // m per millimeter
   define ("M_PER_CM"  , "0.01");          // m per centimeter
   define ("M_PER_IN"  , "0.0254");        // m per inch
   define ("M_PER_FT"  , "0.3048");        // m per foot
   define ("M_PER_YD"  , "0.9144");        // m per yard
   define ("M_PER_M"   , "1");             // m per meter
   define ("M_PER_KM"  , "1000");          // m per kilometer
   define ("M_PER_MI"  , "1609.344");      // m per mile (statute)
   define ("M_PER_NMI" , "1852");          // m per nautical mile
   define ("M_PER_LS", "299792458");       // m per Light Second

   define ("KM_PER_NMI", "1.852");         // km per nautical mile
   define ("KM_PER_MI" , "1.609344");      // kilometers per statute mile

// -----------------------------
// Define the Astronomical Unit.

   define ("M_PER_AU" ,  "149597870700"); // NASA/IAU/DE430 as of 2012
   define ("KM_PER_AU",  "149597870.700");
   define ("MI_PER_AU",  "92955807.273");

// -----------------------------------------------------
// Derived factors based on the above adopted standards.
// The AU used is the IAU value and the year value used
// is the standard Julian year of exactly 365.25 days.

   define ("M_PER_LY"  , "9460730472580800"); // EXACT
   define ("M_PER_PC"  , "30856775813057289.532915512381557");
   define ("AU_PER_LY" , "63241.0770880709446742");
   define ("AU_PER_PC" , "206264.8062470963551565");
   define ("LY_PER_PC" , "3.2615637769712137");

   define ("LT_1AU"    , "0.00577551830441213");

// ----------------------------------
// DEFINE SOME WEIGHT/MASS STANDARDS.

   define ("KG_PER_OZ" , "0.028349523125"); // EXACT
   define ("KG_PER_LB" , "0.45959237");     // EXACT
   define ("G_PER_OZ"  , "28.349523125");   // EXACT

   define ("N_PER_LB"  , "4.4482216152605");

// ------------------------------------------
// LENGTH OF JULIAN CALENDAR YEAR IN SECONDS.
// EXACTLY 365.25 DAYS.

   define ("S_PER_JYR", "31557600");

// ----------------------------------
// DEFINE NUMBER OF SECONDS IN 1 DAY.

   define ("S_PER_DAY" , "86400");

// ---------------------------------------------
// LENGTH OF GREGORIAN CALENDAR YEAR IN SECONDS.
// EXACTLY 365.2425 DAYS.

   define ("S_PER_GYR", "31556952");

// ---------------------------
// Define Planck constant (h).
// 6.62606896E-34

   define ('PLANCK_h', '0.000000000000000000000000000000000662606896'); //  J/s

// -----------------------------------------
// Define Stefan-Boltzmann constant (sigma).
//
// Radiated energy = sigma * K**4 Watts per sq. meter per second
// 5.670373E-8
// Where K = temperature in kelvins

   define ('STEF_BOLT_sigma', '0.00000005670373'); // Watts/sq. meter/k^4

// ------------------------------
// Define Boltzmann constant (k).
//
// k = Joules / deg Kelvin

   define ('BOLTZ_k', '0.000000000000000000000013806504'); //  J/s/K = Watts/K


// ===================================================
// Define ALL available solar system body names. These
// values will be GLOBAL to all function modules.  All
// the fields are padded to a width of 10 characters
// with trailing white space, for uniformity.

   define ('BODY_NAMES', 'Sun       Moon      Mercury   Venus     Mars      Jupiter   Saturn    Uranus    Neptune   Pluto');
   define ('ASTEROID_NAMES', 'Ceres     Cybele    Davida    Eunomia   Europa    Flora     Hebe      Hygiea    InteramniaIris      Juno      Metis     Pallas    Psyche    Vesta');
   define ('MONTH_NAMES', 'January   February  March     April     May       June      July      August    September October   November  December');
   define ('DOW_NAMES', 'Sunday    Monday    Tuesday   Wednesday Thursday  Friday    Saturday');



// FUNCTIONS
// -------------------------------------------------------------------------
// Strips all leading, trailing and internal white space from within a text
// string.  All occurrences of multiple spaces within the text string will
// be replaced by single spaces and all leading and trailing white space
// and NewLine end-codes will also be removed.
//
// This helps facilitate reading sequential space-delimited items from
// a given text-data string by normalizing all spacing between words
// to a single space, for easier use with the preg_split() function.

   function Strip_WSpace ($TextString)
{
   return preg_replace("/\s+/", " ", trim($TextString));
}




/*
   -------------------------------------------------------------------------
   This function simply swaps in place, the
   contents of two simple variable arguments.
*/

   function Swap (&$var1, &$var2)
 {
   $w=$var1;  $var1=$var2;  $var2=$w;
 }




/*
   Sort an alpha or numeric table given as a text string argument
   delimited by NewLine codes.

   Numerical sort is in ascending order from the left and assumes that
   all strings begin with a number and end with a NewLine (\n) code.

   Otherwise, it performs a simple alphabetic text line sort.
*/

   function Sort_Table ($TableTextString)
{
   $wArray = preg_split("[\n]", RTrim($TableTextString, "\n"));
   $wCount = count($wArray);

// Sort table numerically from left.
   sort($wArray);

// Reconstruct and return table as text string.
   $w = '';
   for ($i=0;   $i < $wCount;   $i++)
   {
   $CurrLine = RTrim($wArray[$i], "\n");
   $w .= "$CurrLine\n";
   }
   return RTrim($w, "\n");

} // End of  Sort_Table(...)





/*
   ------------------------------------------------------------------
   Pads a text string with any padding character (default = space) on
   the left or right end to make XPadCount chracters in total length.
*/
   function LR_X_Pad ($TextString, $LorRStr='L', $XPadChar=' ', $XPadCount=0)
{
   $wLorRStr   = (strtoupper(trim($LorRStr)) == 'L')? 'L':'R';
   $wTextStr   = substr($TextString, 0, $XPadCount);
   $wXPadChar  = substr($XPadChar, 0,1);
   $wXPadCount = intval($XPadCount);

   if ($wLorRStr == 'L')
  {
   $w = str_repeat($wXPadChar, $wXPadCount) . $wTextStr;
   $w = substr($w, -$wXPadCount);
  }
   else
  {
   $w = substr($wTextStr, 0, $wXPadCount) . str_repeat($wXPadChar, $wXPadCount);
   $w = substr($w, 0, $wXPadCount);
  }

   return $w;
}







/*
   This function rotates a plain text string by n characters.

   -N = Rotate N characters left
   +N = Rotate N characters right
*/


   function str_rotate($TextString, $RotNumChars=1)

   {
   $TextStr = str_replace("\n", '', $TextString);

   $L = strlen($TextStr);
   $N = intval($RotNumChars);  if ($N == 0){return $TextString;}
   $n = abs($N) % $L;

   if ($N < 0)
  {

// Rotate left by (-n) characters.
   $LeftStr  = substr($TextStr, 0, $n);
   $RightStr = substr($TextStr, $n, $L);
   return $RightStr.$LeftStr;
  }

// Rotate right by (+n) characters.
   $LeftStr  = substr($TextStr, -$n);
   $RightStr = substr($TextStr, 0, $L - $n);
   return $LeftStr.$RightStr;

} // End of  str_rotate(...)



/*
   These two functions padd a string on the left or right ends
   with any chosen padding character (default = space).


   TextString = String to be padded
   PadLen     = Padding length

   PadChar    = Padding character to use (default = space).
                For numbers, a zero can be used instead of a space.

*/
   function L_Pad_Text ($TextString, $PadLen=0, $PadChar=' ')
{
   return substr(str_repeat(substr($PadChar, 0,1), $PadLen).$TextString, -$PadLen);
}


   function R_Pad_Text ($TextString, $PadLen=0, $PadChar=' ')
{
   return $TextString.substr(str_repeat(substr($PadChar, 0,1), $PadLen), 0,$PadLen);
}





/*
   ========================================================================
   Generates a string of any number of random digits in any given base
   from 2 to 36.

   All letters (A to Z) used as digits will be returned in uppercase.

   ERRORS:
   On error, FALSE is returned for invalid argument(s).
   ========================================================================
*/

   function Rand_Digits_Base_B ($NumDigits=1, $BaseB=10, $LZeroSuppressed=FALSE)
{

// Read arguments.
   $N = intval($NumDigits);
   $B = intval($BaseB);

// Check for invalid argument(s).
   if ($B < 2 or $B > 36 or $N < 1) {return FALSE;}

// Define digits spectrum for the given base.
   $DigitsSpectrum = substr("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ",0,$B);

// Construct an N-digit random sequence in base B, one digit at
// a time, using a random 50/50 digit alternation method. There
// will be at least one random base B digit returned.
   $RandomDigits = "";

   for ($i=0;   $i < $N;   $i++)
  {
   if (mt_rand() % 2 == 0)
  {$RandomDigits .= substr($DigitsSpectrum, mt_rand(0,$B-1),1);}
   else
  {$RandomDigits = substr($DigitsSpectrum, mt_rand(0,$B-1),1) . $RandomDigits;}
  }

// If LZeroSuppressed flag === TRUE, then leftmost digit of returned
// random digits string must NOT be zero.  If it is, then another
// random non-zero digit will be attached to the left of it and
// the rightmost digit dropped.  The result will be a normal
// integer value with the required number of digits, but
// never starting with 0 (zero) as the first digit.
   if ($LZeroSuppressed === TRUE)
  {
   $RandomDigits = substr($DigitsSpectrum, mt_rand(1,$B-1),1) . $RandomDigits;
  }
   return substr("$RandomDigits", 0, $N);

} // End of  Rand_Digits_Base_B(...)




   function Rand_Int_Enc_Date()
{
   list ($m, $d, $yyyy) =
   preg_split("[\/]", JDtoGregorian(mt_rand(2305448, 2524958)));
   $mn = sprintf("%02d", $m);
   $dn = sprintf("%02d", $d);
   return "$yyyy$mn$dn";

} // End of  Rand_Int_Enc_Date()



   function Rand_Date_String()
  {
   $w = trim(JD_Z_to_DT (mt_rand(2305448, 2524958), '+00:00', 2));
   $DateStr = substr($w, 0,11);
   $DoWStr  = substr($w, -3);

   return "$DateStr  $DoWStr";

  } // End of  Ran_Date_String()








   function Rand_Time_String()
  {
   $hh = sprintf("%02d", mt_rand(0, 24));
   $mm = sprintf("%02d", mt_rand(0, 59));
   $ss = sprintf("%02d", mt_rand(0, 59));

   return "$hh:$mm:$ss";

   } // End of  Rand_Time_String()





   function Rand_Date_Time_String()
  {
   $RandDateStr = Rand_Date_String();
   $DoWStr      = substr($RandDateStr, -3);
   $RandDateStr = substr($RandDateStr, 0,11);
   $RandTimeStr = Rand_Time_String();

   return "$RandDateStr  $DoWStr  $RandTimeStr";

  } // End of  Rand_Date_Time_String()




   function Rand_Decimal($NumDigits=1, $RandSignFlag=FALSE)
  {
   $RandSign = '';
   $NumDigits = floor($NumDigits);
   $RandSignFlag = ($RandSignFlag === TRUE)? TRUE:FALSE;

   if ($RandSignFlag === TRUE)
      {
       $RandSign = (mt_rand() % 2 == 0)? '-':'+';
      }

   return $RandSign.'0.'.Rand_Digits_Base_B($NumDigits);

  } // End of  Rand_Decimal()









// ========================================
// Basic trigonometric functions for degree
// aguments rather than radians.

   function Sind($AngDMS)
  {
   return Sin(deg2rad(DMS_to_Deg($AngDMS)));
  }

   function Cosd($AngDMS)
  {
   return cos(deg2rad(DMS_to_Deg($AngDMS)));
  }

   function Tand($AngDMS)
  {
   return Tan(deg2rad(DMS_to_Deg($AngDMS)));
  }

   function ASind($x)
  {
   return rad2deg(asin(floatval($x)));
  }

   function ACosd($x)
  {
   return rad2deg(acos(floatval($x)));
  }

   function ATand ($x)
  {
   return rad2deg(atan(floatval($x)));
  }

   function ATand2 ($y, $x)
  {
   $w = rad2deg(atan2(floatval($y), floatval($x)));

   return $w + (($w < 0)? 360:0); // 0 to +360 degrees
  }



/*
============================================================
TEMPERATURE SCALE INTERCONVERSION FUNCTION.

This function converts from any given temperature
scale into the equivalent temperature on any one
of four other recognized (F,C,K,R) scales.

The scale symbols are:
F = Fahrenheit
C = Celsius
K = Kelvin  = Degrees Celsius above absolute zero.
R = Rankine = Degrees Fahrenheit above absolute zero.

The scale symbols are NOT case sensitive.

The conversion is performed by first converting the
input temperature value into its equivalent on the K
scale, then the K scale value is converted into the
given output scale.

-------
EXAMPLE
To convert from 123.45 C into its equivalent
on the R scale:

print Temp_To_Temp("-123.45 C", "R"); // Result = 269.46


------
ERRORS
An error returns FALSE if a scale symbol is not recognized.
============================================================
*/


   function Temp_to_Temp ($FromTempStr, $ToScaleStr)
{
// -------------------------------------
// Read input argument value and scales.

   $w = trim($FromTempStr);   if ($w == '') {$w = "0";}
   $T = floatval($w);

   $FromScale = strtoupper(trim(substr($w, -1)));
   $ToScale = strtoupper(trim($ToScaleStr));

// ---------------------------------------------------
// Convert input ($FromScale) into K scale equivalent.

   switch ($FromScale)
  {
   case "F" : $K = (5 * ($T - 32) / 9) + 273.15; break;
   case "C" : $K = $T + 273.15; break;
   case "K" : $K = $T; break;
   case "R" : $K = (5 * ($T - 491.67) / 9) + 273.15; break;
   default  : $K = FALSE; break; // On ERROR
  }

// --------------------------
// ERROR if invalid FromScale

   if ($K === FALSE) {return $K;}

// --------------------------------------------------
// Convert K value into output ($ToScale) equivalent.

   switch (strtoupper(trim($ToScale)))
  {
   case "F" : $T = (9 * ($K - 273.15) / 5) + 32; break;
   case "C" : $T = $K - 273.15; break;
   case "K" : $T = $K; break;
   case "R" : $T = (9 * ($K - 273.15) / 5) + 491.67; break;
   default  : $T = FALSE; break;
  }

   return $T;

} // End of Temp_To_Temp(...)




/*
 ===========================================================================
 This function interconverts between any of sixteen length or
 distance units.  The unit symbols are NOT case sensitive.

 ----------
 ARGUMENTS:
 $LengthAndUnits = Numerical value and units symbol of length to convert.
                   NOTE: There MUST be at least one space between the
                   number value and the units symbol in the string.
                   The unit symbols are NOT case sensitive.

 $ToUnits = The units symbol corresponding to the computed output value.

 -------
 ERRORS:
 On error FALSE is returned.
 An error occurs if the length value is non-numeric or either
 of the input or output length units symbols are unrecognized.

 ------------------------------------------------------------
 THE 16 LENGTH UNITS SYMBOLS RECOGNIZED BY THIS FUNCTION ARE:

 SYMBOL      UNITS

   A         Angstroms
   nm        Nanometers
 um or u     Micrometers (or Microns)
   mm        Millimeters
   cm        Centimeters
   in        Inches
   ft        Feet
   yd        Yards
   m         Meters
   km        Kilometers
   mi        Miles (statute)
   nmi       Nautical miles
   LS        Light Seconds
   AU        Astronomical Units
   LY        Light Years (Based on Julian year of 365.25 days)
   pc        Parsecs

   ===========================================================================
*/

   function Length_to_Length ($LengthAndUnits, $ToUnits)
{
// Define conversion constants for internal use only.
// These constants could also be defined externally
// for universal use by any programs that need them.
// This function was designed to be transportable.
   $M_PER_A   = 0.0000000001;
   $M_PER_NM  = 0.000000001;
   $M_PER_UM  = 0.000001;
   $M_PER_MM  = 0.001;
   $M_PER_CM  = 0.01;
   $M_PER_IN  = 0.0254;
   $M_PER_FT  = 0.3048;
   $M_PER_YD  = 0.9144;
   $M_PER_M   = 1;
   $M_PER_KM  = 1000;
   $M_PER_MI  = 1609.344;
   $M_PER_NMI = 1852;
   $CLIGHT    = 299792458;
   $M_PER_AU  = 149597870700;
   $M_PER_LY  = 9460730472580800;
   $M_PER_PC  = 30856775876793114.6633;

// Parse input value and separate the numerical value from the symbol.
// There MUST be at least one space between the number and the symbol.
// FALSE is returned on error if no space character detected at all or
// the given numeric value is not a valid number.
   $w = trim($LengthAndUnits); if (strpos($w, " ") === FALSE){return FALSE;}

   list($x, $FromUnits) = preg_split("[ ]", preg_replace("/\s+/", ' ', $w));

   if (!is_numeric($x)) {return FALSE;}

// Get input units symbol.  Unit symbols are NOT case sensitive.
   $FromUnitsSymbol = strtoupper(trim($FromUnits));

// Determine which conversion factor to use
// according to the units symbol argument.

   switch ($FromUnitsSymbol)
  {
   case "A"  : $f = $M_PER_A;   break;
   case "NM" : $f = $M_PER_NM;  break;
   case "UM" : $f = $M_PER_UM;  break;
   case "U"  : $f = $M_PER_UM;  break;
   case "MM" : $f = $M_PER_MM;  break;
   case "CM" : $f = $M_PER_CM;  break;
   case "IN" : $f = $M_PER_IN;  break;
   case "FT" : $f = $M_PER_FT;  break;
   case "YD" : $f = $M_PER_YD;  break;
   case "M"  : $f = $M_PER_M;   break;
   case "KM" : $f = $M_PER_KM;  break;
   case "MI" : $f = $M_PER_MI;  break;
   case "NMI": $f = $M_PER_NMI; break;
   case "LS" : $f = $CLIGHT;    break;
   case "AU" : $f = $M_PER_AU;  break;
   case "LY" : $f = $M_PER_LY;  break;
   case "PC" : $f = $M_PER_PC;  break;
   default   : $f = FALSE;      break;
  }

// Error if conversion factor is FALSE, which
// means that the units symbol was not found
// in the table of valid symbols.
   if ($f === FALSE) {return $f;}

// Otherwise, compute the equivalent meters value
// by multiplying the value of the argument by
// the conversion factor $f.
   $m = $x * $f;

// *********************************************************
// AT THIS POINT INPUT VALUE HAS BEEN CONVERTED INTO METERS.
// NEXT STEP IS TO CONVERT THE METERS INTO THE OUTPUT UNITS.
// *********************************************************

// Read the output units symbol argument.  The unit symbols
// are NOT case sensitive.
   $ToUnitsSymbol = strtoupper(trim($ToUnits));

// Determine the conversion factor to use from the units
// symbol argument.

   switch ($ToUnitsSymbol)
  {
   case "A"  : $f = $M_PER_A;   break;
   case "NM" : $f = $M_PER_NM;  break;
   case "UM" : $f = $M_PER_UM;  break;
   case "U"  : $f = $M_PER_UM;  break;
   case "MM" : $f = $M_PER_MM;  break;
   case "CM" : $f = $M_PER_CM;  break;
   case "IN" : $f = $M_PER_IN;  break;
   case "FT" : $f = $M_PER_FT;  break;
   case "YD" : $f = $M_PER_YD;  break;
   case "M"  : $f = $M_PER_M;   break;
   case "KM" : $f = $M_PER_KM;  break;
   case "MI" : $f = $M_PER_MI;  break;
   case "NMI": $f = $M_PER_NMI; break;
   case "LS" : $f = $CLIGHT;    break;
   case "AU" : $f = $M_PER_AU;  break;
   case "LY" : $f = $M_PER_LY;  break;
   case "PC" : $f = $M_PER_PC;  break;
   default   : $f = FALSE;      break;
  }

// Error if conversion factor is FALSE, which
// means that the units symbol was not found
// in the valid symbols table.
   if ($f == FALSE) {return FALSE;}

// Otherwise, return the equivalent of the given
// meters value by dividing the value of the
// argument by the conversion factor $f.
   return $m / $f;

} // End of  Length_To_Length(...)







/*
   ------------------------------------------------------------------
   This function computes the arbitrary-precision Nth root of a given
   numerical string X, rounded to any specified number of decimals.

   ARGUMENTS:
   N = Root = Integer 2, 3, 4, ..., etc.
   X = Numerical argument string for which we seek the Rth root

   This function is based on the master root iteration formula.

   MASTER ROOT ITERATION FORMULA

   Let
   N = Root to iterate (1, 2=Square root, 3=Cube root, etc. ...)
   a = Current approximation to Rth root of X
   b = Next approximation descended from a



   Given (N, X, a):

   n = N-1
   START:
   b = (a + X/(a^n)) / n

   For next approximation
   a = b
   Repeat from START until required accuracy reached.

   -----------------------------------------------------------------
   NumDecimals = Number of decimals at which to round off the result
                 if exact resolution is not possible.

   ERRORS:
   NO special error checking is done in this version of the function.
   ------------------------------------------------------------------
*/

   function bcNth_Root_Of_X ($N, $X, $decimals=16)
{
   $a = sprintf("%1.16f", pow($X, 1/$N)); // Compute first approximation
   $n = $N - 1;
   $d = abs(intval($decimals));
   $D = $d + 8;

   for ($i=1;  $i <= 50;   $i++)
  {
   $b = bcdiv(bcadd(bcmul($n,$a,$D),bcdiv($X, bcpow($a,$n,$D),$D),$D),$N,$D);

   if (bccomp($a,$b,$D) == 0) {break;} else {$a = $b;}
  }
   return rtrim(rtrim(bcadd($b, "0." . str_repeat(0,$d) . "5", $d), "0"), ".");
}





/*
   ===========================================================================
   This function returns the arbitrary-precision factorial of any non-negative
   integer value.  This function should be used with caution, since factorial
   values can be extremely large for relatively small arguments.  It may take
   several seconds to compute extremely large factorials (e.g. > 10000!), so
   a safety limit of 9999 is built in, but it can be easily removed.

   Input argument must be a positive integer.  Any decimal argument will be
   silently truncated to an integer, without rounding, rather than causing
   an error.

   ERRORS:
   FALSE is returned on error, if argument is non-numeric, negative or the
   argument exceeds the maximum set limit (default = 9999).
   ===========================================================================
*/

   function bcN_Factorial ($N=0)
{

// Error if argument N is non-numeric, negative or > 9999.
   if (!is_numeric($N) or $N < 0 or $N > 9999) {return FALSE;}

// Truncate any decimal argument to an
// integer value, without rounding.
   $n = bcadd($N, 0);

// Execute loop to compute N factorial product (P).
   $P=1;  for ($i=1;  $i <= $n;  $i++) {$P = bcmul($P, $i);}

// Done.
   return $P;
}




/*
  ========================================================================
  This function returns the permutations of (N) items taken (R) at a time.
  Like the digits in a number, the sequential order of items is important.

  Input arguments must be positive integers.  Any decimal argument will be
  truncated to an integer, without rounding, rather than returning an error.

   ERRORS:
   FALSE is returned on error, if either argument is non-numeric or out
   of valid range.
  ========================================================================
*/

  function bcPerm_N_R ($IntNStr, $IntRStr)
{
// Read numeric string arguments. Must be
// non-negative integer strings only. Any
// decimal part will be ignored and lost.
   $N = trim($IntNStr);
   $R = trim($IntRStr);

// Error if either argument is non-numeric.
   if (!is_numeric($N) or !is_numeric($R)) {return FALSE;}

// Error if either argument is out of valid range.
   if ($N < 0 or $R < 0 or $R > $N) {return FALSE;}

// Truncate argument(s) to integer value(s) without rounding.
   $N = bcadd($N, 0);  $R = bcadd($R, 0);

// Execute loop to compute P(N, r).
   $P=1;  for ($i=0;  $i < $R;  $i++) {$P = bcmul($P, $N-$i);}

// Done.
  return $P;
}




/*
  ========================================================================
  This function returns an arbitrary precision count of all total possible
  combinations or mixtures of (N) items taken (R) at a time.  Like a hand
  dealt in a poker game, the sequential order in which the cards were
  received is NOT important, but only which cards are held in the hand,
  regardless of their arrangement or order.

  ERRORS:
  FALSE is returned on error, if either argument is non-numeric or out
  of valid range.
  ========================================================================
*/

  function bcComb_N_R ($N, $R)
{
// Error if either argument is non-numeric.
   if (!is_numeric($N) or !is_numeric($R)) {return FALSE;}

// Truncate argument(s) to integer value(s) without rounding.
   $N = bcadd($N, 0);
   $R = bcadd($R, 0);

// Error if either argument is out of valid range.
   if ($N < 0 or $R < 0 or $R > $N) {return FALSE;}

// Execute loop to compute C(N, r).
   $C=1;  for ($i=0;  $i < $N-$R;  $i++) {$C = bcdiv(bcmul($C, $N-$i), $i+1);}

// Done.
   return $C;
}












/*
  ========================================================================
  This function returns the GCD (Greatest Common Divisor) of any two
  arbitrary-precision integers.  The GCD value will always be a
  positive value from 1 up to the lesser of the absolute values
  of the two integer arguments (A, B).

  Any numerical signs () are ignored.

  ERRORS:
  FALSE is returned on error if either argument is non-numeric.
  ========================================================================
*/

/*
   =========================================================================
   This function returns the greatest common divisor for any two integer
   value strings.  The numerical signs of the numbers are ignored.

   Arbitrary-precision arithmetic is used, so the integers should be given
   as strings, like "1234", so as not to be mistaken for standard floating-
   point values.

*/

   function bcGCD ($IntAStr, $IntBStr)
{
// Read integer string arguments.
   $A = trim($IntAStr);
   $B = trim($IntBStr);

// ERROR if either argument is non-numeric.
   if (!is_numeric($A) or !is_numeric($B)) {return FALSE;}

// Remove any negative signs and work with absolute values.
   $A = str_replace("-", "", $A);
   $B = str_replace("-", "", $B);

// Truncate any non-integer to an integer value WITHOUT rounding.
// This means that anything following the decimal point will
// simply be ignored and lost without rounding the integer part.
   $A = bcadd($A,0);     $B = bcadd($B,0);

// Return 1 if either integer equates to zero.
   if (bccomp($A,0) == 0 or bccomp($B,0) == 0) {return 1;}

// Swap (A,B) values, if necessary, to ensure that B >= A initially.
   if (bccomp($B,$A) < 0) {$x = $B;     $B = $A;     $GCD = $x;}
   else {$GCD = $A;}

// Compute and return the final GCD of the given arguments.
   while ($B != 0) {$x = $B;     $B = bcmod($GCD, $B);     $GCD = $x;}

   return $GCD; // Done

} // End of  bcGCD(...)



/*
   ========================================================================
   This PHP function performs arbitrary-precision
   integer fraction addition without reduction.

   (A/B) + (C/D) = E/F
   ========================================================================
*/

   function bcAdd_Frac ($A_B, $C_D)
{
   list($A,$B) = preg_split("[\/]", preg_replace("/\s+/", '', trim($A_B)));
   list($C,$D) = preg_split("[\/]", preg_replace("/\s+/", '', trim($C_D)));

   $E = bcadd(bcmul($A,$D), bcmul($B,$C));
   $F = bcmul($B,$D);

   return "$E/$F";
}





/*
   ========================================================================
   This PHP function performs arbitrary-precision
   integer fraction subtraction without reduction.

   (A/B) - (C/D) = E/F
   ========================================================================
*/

   function bcSub_Frac ($A_B, $C_D)
{
   list($A,$B) = preg_split("[\/]", preg_replace("/\s+/", '', trim($A_B)));
   list($C,$D) = preg_split("[\/]", preg_replace("/\s+/", '', trim($C_D)));

   $E = bcsub(bcmul($A,$D), bcmul($B,$C));
   $F = bcmul($B,$D);

   return "$E/$F";
}



/*
   ========================================================================
   This PHP function performs arbitrary-precision
   integer fraction multiplication without reduction.

   (A/B) * (C/D) = E/F
   ========================================================================
*/

   function bcMul_Frac ($A_B, $C_D)
{
   list($A,$B) = preg_split("[\/]", preg_replace("/\s+/", '', trim($A_B)));
   list($C,$D) = preg_split("[\/]", preg_replace("/\s+/", '', trim($C_D)));

   $E = bcmul($A,$C);     $F = bcmul($B,$D);

   return "$E/$F";
}




/*
   ========================================================================
   This function performs arbitrary-precision
   integer fraction division without reduction.

   (A/B) / (C/D) = (E/F)
   ========================================================================
*/

   function bcDiv_Frac ($A_B, $C_D)
{
   list($A,$B) = preg_split("[\/]", preg_replace("/\s+/", '', trim($A_B)));
   list($C,$D) = preg_split("[\/]", preg_replace("/\s+/", '', trim($C_D)));

   $E = bcmul($A,$D);     $F = bcmul($B,$C);

   return "$E/$F";
}

/*
   ========================================================================
   This function reduces arbitrary-precision integer fraction (A/B)
   to its lowest terms (a/b), if possible.

   Argument must be an integer fraction string in 'A/B' format.

   ERRORS:
   No special error checking is done.
   ========================================================================
*/

   function bcReduce_Frac ($A_B)
{
// -----------------------------------------------
// Extract the individual fraction elements (A, B)
// from the input argument string ("A/B").

   list($A,$B) = preg_split("[\/]", str_replace(' ','', trim($A_B)));

// -------------------------------------------------
// To find the GCD, we work with the absolute values
// of E and F, calling those working values wE and
// wF respectively.

   $wA = bcmul(((bccomp($A, 0) < 0)? -1:1), $A);
   $wB = bcmul(((bccomp($B, 0) < 0)? -1:1), $B);

   if (bccomp($wA,$wB) > 0) {$w=$wA;   $wA=$wB;   $wB=$w;}

// -------------------------------------
// Equate 1st approximation of GCD to wA

   $GCD = $wA;

// ------------------------------------------------
// Perform Euclid's algorithm loop while (wB != 0).
// The value of wB decreases after each cycle and
// eventually reaches zero, at which point we have
// reached the GCD value.  This will always equate
// to at least 1.

   while (bccomp($wB, 0) != 0)
         {$w=$wB;     $wB=bcmod($GCD, $wB);     $GCD=$w;}

// ------------------------------------------
// Compute and return reduced fraction (a/b).
// If GCD == 1, then the fraction cannot be
// reduced and is returned unchanged.

   $a = bcdiv($A, $GCD);
   $b = bcdiv($B, $GCD);

   return "$a/$b";

} // End of  bcReduce_Frac (...)




/*
   This function adds a string of BC decimal values together.
   The individual values are space delimited integers or
   fractional values.

   RetModeStr = Return mode (D = Decimal fraction  or  I = Integer fraction)

   String of Numbers = "1 2 1/3 4 5 6 7 8 9 10 11"
   decimals = 32
   sum  = 63.33333333333333333333333333333333

*/
   function bcAdd_Num_String ($StringOfNumbers, $RetModeStr="Dec", $decimals=16)
{
// ------------------------------------------------------------
// Read argument string of numerical expressions.  These can be
// integers or fractions, including decimal values.

   $w = preg_replace("/\s+/", " ", trim($StringOfNumbers));

// ------------------------------------------------
// Create 'rounding mask' to apply for rounding off
// the final result to the given number of decimals.

   $RoundingMaskStr = "0." . str_repeat("0", $decimals) . "5";

// -------------------------------------------------
// Read return mode argument and force to uppercase.
// Only 1st character is read (Dec or Int fraction).

   $RModeStr = substr(strtoupper(trim($RetModeStr)), 0,1);

// -----------------------------------
// Store expressions in working array.

   $NumsArray = preg_split("[ ]", $w);

// ----------------------
// Count the expressions.

   $NumsCount = count($NumsArray);

// ----------------------------------------------
// Cycle through the expressions, converting them
// all into exactly equivalent integer fractions.

   $sum = "0/1";

   for($i=0;   $i < $NumsCount;   $i++)
  {
// Get current number from work array and then
// convert any decimal fraction into integer
// fraction.
   $w = bcDec_to_Int_Frac ($NumsArray[$i]);

// Split current sum into A,B and current
// fraction to C,D and
   list($A,$B) = preg_split("[\/]", $sum);
   list($C,$D) = preg_split("[\/]", $w);

// Compute new A/B to replace previous A/B and
// repeat until all numbers have been summed.
   $AD = bcmul($A,$D);   $BC = bcmul($B,$C);   $BD = bcmul($B,$D);

   $A = bcadd($AD,$BC);  $B = $BD;

   $sum = "$A/$B"; // Dec_to_Int_Frac("$A/$B");
  }

// Reduce fraction to its lowest
// terms if possible.
// $sum = Dec_to_Int_Frac("$A/$B");

 $NegSign = "";


  if ($RModeStr == "D")
 {
  list($A,$B) = preg_split("[\/]", $sum);

  $sum = bcdiv($A,$B, $decimals);

// Temporarily remove any negative sign (-).  It will
// be re-attached to the returned result.
   if (substr($sum,0,1) == "-")
      {
       $NegSign = "-";   $sum = substr($sum, 1, strlen($sum));
      }

// Round off absolute sum value to given number of decimals
// and then trim off any redundant zeros and/or decimal point.
   $sum = rtrim(rtrim(bcadd($sum, $RoundingMaskStr, $decimals), "0"), ".");
 }
// Reattach any negative sign.  Done.
   return $NegSign . $sum;

} // End of  bcAdd_Num_String(...)







/*
   This function multiplies a string of BC decimal values together.
   The individual values are space delimited integers or fractional
   values in any combination.

   RetModeStr = Return mode (D = Decimal fraction  or  I = Integer fraction)

   PROD

    A     C     A*C     E
   --- * --- = ----- = ---
    B     D     B*D     F


   1/3.15  2  2.90/1.3  4  5  6  7  8  9  10  13  10  11
   =
   1/3.15 * 2 * 2.90/1.3 * 4 * 5 * 6 * 7 * 8 * 9 * 10 * 13 * 10 * 11

*/
   function bcMul_Num_String ($StringOfNumbers, $RetModeStr="Dec", $decimals=16)
{
// Read argument string of numerical expressions.  These can be
// integers or fractions, including decimal values.
   $w = preg_replace("/\s+/", " ", trim($StringOfNumbers));

// Create 'rounding mask' to apply for rounding off
// the final result to the given number of decimals.
   $RoundingMaskStr = "0." . str_repeat("0", $decimals) . "5";

// Read return mode argument and force to uppercase.
// Only 1st character is read (Dec or Int fraction).
   $RModeStr = substr(strtoupper(trim($RetModeStr)), 0,1);

// Store expressions in working array.
   $NumsArray = preg_split("[ ]", $w);

// Count the expressions.
   $NumsCount = count($NumsArray);

// Cycle through the expressions, converting them
// all into exactly equivalent integer fractions.
   $prod = "1/1";

   for($i=0;   $i < $NumsCount;   $i++)
  {
// Get current number from work array and then
// convert any decimal fraction into integer
// fraction.
   $w = Dec_to_Int_Frac ($NumsArray[$i]);

// Split current product into A,B and current
// working number (w) string into to C,D
   list($A,$B) = preg_split("[\/]", $prod);
   list($C,$D) = preg_split("[\/]", $w);

// Compute new A/B to replace previous A/B and
// repeat until all numbers have been multiplied.
   $AC = bcmul($A,$C);   $BD = bcmul($B,$D);

   $A = $AC;  $B = $BD;

   $prod = "$A/$B";
  }

// Reduce fraction to its lowest
// terms if possible.
 $prod = Dec_to_Int_Frac("$A/$B");

 $NegSign = "";


  if ($RModeStr == "D")
 {
  list($A,$B) = preg_split("[\/]", $prod);

  $prod = bcdiv($A,$B, $decimals);

// Temporarily remove any negative sign (-).  It will
// be re-attached to the returned result.
   if (substr($prod,0,1) == "-")
      {
       $NegSign = "-";   $prod = substr($prod, 1, strlen($prod));
      }

// Round off absolute product value to given number of decimals
// and then trim off any redundant zeros and/or decimal point.
   $prod = rtrim(rtrim(bcadd($prod, $RoundingMaskStr, $decimals), "0"), ".");
 }
// Reattach any negative sign.  Done.
   return $NegSign . $prod;

} // End of  bcMul_Num_String(...)







/*
   ========================================================================
   This function will convert an arbitrary precision positive base 10
   X value into its equivalent in any base from 2 to 36.  The X value
   is NOT restricted to integers and also allows fractional values.

   ARGUMENTS:
   $X10 = Base 10 numeric string to be to converted
          into its BaseB equivalent.

   $BaseB = Base (2 to 36) in which to interpret input digit string.

   ERRORS:
   On error FALSE is returned.  An error occurs if either
   of the arguments is non-numeric or out of valid range.

   This is a level 2 function, dependent on:
   bcBase10_Int_To_BaseB (...)

   ========================================================================
*/

   function bcBase10_To_BaseB ($X10, $BaseB, $TruncationLimit=0)
{
   $p = 1000; // Decimals precision control factor (Don't ask).

   $q =  $TruncationLimit; // Decimal truncation limit.

   $X = trim($X10);  $sign = (substr($X,0,1) == "-")? "-" : "";
   $X = str_replace("-","", $X);

   if (strpos($x, ".") === FALSE){$X .= ".0";}

   list($i, $f) = preg_split("[\.]", $X);  $f = "0.$f";

   $IntPartOut = bcBase10_Int_To_BaseB ($i, $BaseB);

   $w = bcmul(bcPow($BaseB, $p), $f);

   $fp = bcBase10_Int_To_BaseB ($w, $BaseB);

   $zeros = str_repeat("0", $p - strlen($fp));

   if ($q > 0) {$fp = substr($fp, 0, $q-0);}

   $FracPartOut = ".$zeros$fp";

   $FracPartOut = rtrim(rtrim($FracPartOut,'0'), '.');

   return "$sign$IntPartOut$FracPartOut";

} // End of  bcBase10_To_BaseB(...)





/*
   ========================================================================
   This function will convert an arbitrary precision number value in
   any base from 2 to 36 into its equivalent in base 10.  It is NOT
   restricted only to integers and applies to fractional values as well.

   ARGUMENTS:
   $BaseBIntStr = Base B numeric string to be to converted
                  into its base 10 equivalent.

   $BaseB = Base of input digit string (2 to 36).

   ERRORS:
   On error FALSE is returned.  An error occurs if either
   of the arguments is non-numeric or out of valid range.

   This is a level 2 function.
   ========================================================================
*/


   function bcX_Base_B_To_Base_10 ($BaseBNumStr, $BaseB)
{

// Compute base 10 decimal equivalent to BaseB fraction.

   $a = 0;
   $b = 1;

   $BaseDigits = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

// Read and copy input arguments.
   $x = trim($BaseBNumStr);

// Remember numerical sign, if any and work with absolute
// value.  The sign is restored to output value.
   $sign = (substr($x, 0,1) != "-")? '' : '-';
   $x = str_replace("-", '', $x);
   $x = str_replace("+", '', $x);

// If argument is an integer value, then attach ".0" to it.
   if(strpos($x, ".") === FALSE) {$x .= ".0";};

// Get only integer part of the BaseB numeric string.
   $xi = substr($x, 0, strpos($x, "."));

// Compute equivalent base 10 integer value.
   $IntegerPartBase10 = bcBase_B_Int_To_Base_10($xi, $BaseB);

// Get only the fractional digits of the BaseB numeric string.
   $xf = substr($x, strpos($x, ".")+1, strlen($x));

// Count BaseB fractional digits.
   $DigitsCount = strlen($xf);

// Cycle through each digit, creating integer fraction terms
// and summing them up to final a/b.

   $out = '';

   $PowB = $BaseB;

   for($i=0;   $i < $DigitsCount;   $i++)
   {
// get digit and convert to base 10 equivalent.
   $c = strpos($BaseDigits, substr($xf, $i, 1));
   $d = $PowB;  $PowB = bcmul($PowB, $BaseB);

// Add to current (a/b).
   $a = bcadd(bcmul($a,$d), bcmul($b,$c));
   $b = bcmul($b,$d);

//   $out .= "$c/$d\n";

   }

// Compute final raw, unreduced (a/b) value.
   $c = $IntegerPartBase10;
   $d = "1";
   $a = bcadd(bcmul($a,$d), bcmul($b,$c));
   $b = bcmul($b,$d);

//   return $out;

   $w = bcreduce_frac("$sign$a/$b");

   $w = preg_replace("/\s+/", '', trim($w)); // Modified vesrion

// If single value, then put over 1.
   if (strpos($w, "/") === FALSE) {$w .= "/1";}

// Get numerator and denominator values.
   list($A,$B) = preg_split("[\/]", $w);

// Compute decimal value equivalent truncated
// at 1000 decimals.
   $w = bcdiv($A,$B, 1000);

   return rtrim(rtrim($w,'0'), '.');

} // End of  bcXBaseB_To_Base10(...)




/*
   ========================================================================
   This boolean function tests to see of the given argument is a valid
   BaseB (2 to 36) number.

   If valid, then TRUE is returned, otherwise FALSE is returned.

   NOTE:
   It is possible for a BaseB number to be valid in more than one base.

   A given number is considered valid only if none of its digits are
   invalid for the given base.  For example, a base 10 number cannot
   contain the digit 'F', but a hexadecimal (base 16) number can.

*/

   function bcIs_Base_B ($XBaseBString, $BaseB)
{
// Read numeric work string and remove any decimal or numerical sign.
   $X = str_replace(".", '', strtoupper(trim($XBaseBString)));
   $X = str_replace("-", '', $X);
   $X = str_replace("+", '', $X);

   $DigitsSpectrum = substr("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ", 0, $BaseB);

// Cycle through every digit.  Invalid if any digit is not found
// in DigitsSpectrum string, so return FALSE.
// Otherwise, TRUE is returned.
   $DigitsCount = strlen($X);
   for ($i=0;   $i < $DigitsCount;   $i++)
       {
        $digit = substr($X, $i, 1);
        if (strpos($DigitsSpectrum, $digit) === FALSE) {return FALSE;}
       }
   return TRUE;

} // End of  bcIs_Base_B(...)




/*
   ========================================================================
   This function will convert an arbitrary precision integer
   in any base from 2 to 36 into its equivalent in base 10.

   ARGUMENTS:
   $XBaseBString = Base B integer string to be converted
            into its base 10 equivalent.

   $FromBase = Base of input digit string (2 to 36).

   ERRORS:
   On error, FALSE is returned if either argument is invalid.

   ========================================================================
*/

   function bcBaseB_Int_To_Base10 ($XBaseBString, $BaseB)
{

// Read input arguments.
   $Xb = strtoupper(trim($XBaseBString));
   $B  = trim($BaseB);

// ERROR if base is outside of range 2 to 36.
   if ($B < 2 or $B > 36) {return FALSE;}

// Remember any numerical sign and work with absolute value.
// The numerical sign will be restored at the end.
   $sign = (substr($Xb, 0,1) == '-')? '-' :'';
   if ($sign == '-') {$Xb = substr($Xb, 1, strlen($Xb));}

// Count digits in base b integer X.
   $XbDigitsCount = strlen($Xb);

// Initialize base 10 summation accumulator
// and initial power (= 1) of the base.
   $X10 = 0;
   $PowB = 1;

// Define the valid digits spectrum for the given base.
   $DigitsSpectrum = substr("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ", 0, $B);

// Perform loop to compute the base 10
// equivalent of the base b integer Xb.
   for ($i=0;   $i < $XbDigitsCount;   $i++)
       {
//      Get current base b digit from Xb.
        $db = substr($Xb, $XbDigitsCount - $i - 1, 1);

//      Get base 10 numerical value (= String index) of digit.
        $d10 = strpos($DigitsSpectrum, $db);

//      If FALSE is returned, it means that the base b digit was
//      not found within the valid base b DigitsSpectrum string.
        if ($d10 === FALSE) {return FALSE;}

//      Compute and accumulate value of current base 10 term.
        $X10 = bcadd($X10, bcmul($PowB, $d10));

//      Update power of base for next cycle.
        $PowB = bcmul($PowB, $B);
       }

// Done.
   return "$sign$X10";

} // End of  bcBaseB_Int_To_Base10(...)



/*
   ========================================================================
   This function will convert an arbitrary precision base 10 integer into
   its equivalent in any given base from 2 to 36.

   ARGUMENTS:
   $x10IntStr = Base 10 integer string to be to converted
                into its Base B equivalent.

   $BaseB = Base of converted output (2 to 36).

   ERRORS:
   On error, FALSE is returned if either argument is non-numeric
   or out of valid range.
   ========================================================================
*/

   function bcBase10_Int_To_BaseB ($X10IntStr, $BaseB)
{
// Error if either argument is non-numeric
// or if base of out of range (2 to 36).

   if (!is_numeric($X10IntStr) or !is_numeric($BaseB)) {return FALSE;}

   if ($BaseB < 2 or $BaseB > 36) {return FALSE;}

// Define available digits spectrum for bases 2 to 36.
   $DigitsSpectrum = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

   $XB = "";
   $X10 = trim($X10IntStr);

// Remember any numerical sign and work with absolute value.
// The numerical sign will be restored at the end.
   $sign = (substr($X10, 0,1) == '-')? '-' :'';
   if ($sign == '-') {$X10 = substr($X, 1, strlen($X10));}

   if (bccomp($X10, 0) == 0) {return "0";} // Handle zero argument.

// Perform base 10 to base B conversion loop, digit-by-digit.
   while (bccomp($X10, 0) > 0)
  {
   $XB = substr($DigitsSpectrum, bcmod($X10, $BaseB), 1) . $XB;
   $X10 = bcdiv($X10, $BaseB);
  }
// Done.
   return "$sign$XB";

}  // End of  bcBase10_Int_To_BaseB(...)




/*
   This function converts a single integer or decimal fraction string
   into the equivalent fraction in pure integer form (A/B).

   NOTE: Numbers must be in standard form, NOT scientific E-notation.

   "12.042/3.71" would become "12042/3710"

   "1.5000/0.17" would become "150/17"

   "1.234" would become "1234/1000"

   "-0.02337/11.7346709" would become "-233700/117346709"

*/
   function bcDec_to_Int_Frac ($FractionString)
{
// Read fraction string argument and remove ALL spaces.
   $FracStr = str_replace(" ", "", trim($FractionString));

// If only a single value, put it over 1 (like A/1).
   if (strpos($FracStr, "/") === FALSE)  {$FracStr = "$FracStr/1";}

// Split fraction into numerator and denominator elements.
   list($numer, $denom) = preg_split("[\/]", $FracStr);

// If integer (no decimal) part, then append .0 to it.
   $numer .= (strpos($numer, ".") === FALSE)? ".0":"";
   $denom .= (strpos($denom, ".") === FALSE)? ".0":"";

// If any decimal numbers start with a sign followed by a decimal point
// or start with only a decimal point, then a zero must be inserted be-
// fore the decimal.  For example, -.125 would become -0.125 instead.
   $numer = str_replace("-.", "-0.", $numer);
   $numer = str_replace("+.", "+0.", $numer);
   if (substr($numer,0,1) == ".") {$numer = str_replace(".", "0.", $numer);}
   $denom = str_replace("-.", "-0.", $denom);
   $denom = str_replace("+.", "+0.", $denom);
   if (substr($denom,0,1) == ".") {$denom = str_replace(".", "0.", $denom);}



// Isolate only the decimal parts of numerator and denominator.
// This means everything from the decimal point and after.
   $NumerDec = strrchr($numer, '.');
   $DenomDec = strrchr($denom, '.');

// Length of longest decimal part determines power of
// 10 to be used as the common multiplier to replace
// the decimal values with integers.
   $Pow10 = bcpow("10", max(strlen($NumerDec)-1, strlen($DenomDec)-1));

// Multiply both numerator and denominator by
// the common power of 10 needed to convert
// them both into integer values.
   $numer = bcmul($Pow10, $numer);
   $denom = bcmul($Pow10, $denom);

// Remove any redundant common zeros from numerator and denominator.
   while(substr($numer, -1) == "0" and substr($denom, -1) == "0")
  {
   $numer = substr($numer, 0, strlen($numer)-1);
   $denom = substr($denom, 0, strlen($denom)-1);
  }

// Reduce to lowest integer terms, if possible.
// Otherwise, finished.
   $A = $numer;
   $B = $denom;

// If negative A or B argument, then convert to absolute
// value. Any numerical signs are irrelevant and removed.
   if (substr($A,0,1) == "-"){$A = substr($A,1,strlen($A));}
   if (substr($B,0,1) == "-"){$B = substr($B,1,strlen($B));}

// Swap working (A, B) values, if necessary,
// so that A > B initially.
   if (bccomp($A, $B) < 0)
      {
       $w   = $B;
       $B   = $A;
       $GCD = $w;
      }
   else
      {
       $GCD = $A;
      }

// Compute the GCD, if any, of the given arguments.  This value
// is used to reduce the integer fraction to its lowest terms.
   while ($B != 0)   {$w = $B;    $B = bcmod($GCD, $B);    $GCD = $w;}

// Now, reduce fraction to lowest terms
// by dividing both terms by their GCD.
   $numer = bcdiv($numer, $GCD);
   $denom = bcdiv($denom, $GCD);

// Done.
   return "$numer/$denom";

} // End of  Dec_to_Int_Frac(...)



















/*
   ========================================================================
   This function performs LaGrange interpolation within an XY data table.

   ========================================================================
*/

   function LaGrange_Interp ($XYDataTable, $xArg)
{

// -----------
// Initialize.

   $XDataStr = $YDataStr = '';

// --------------------------------------------------------------
// Read and split XY data pairs into a work array.  In the array,
// even number indices = X-data, odd number indices = Y-data.

   $XY = preg_split("[ ]", preg_replace("/\s+/", ' ', trim($XYDataTable)));

// ---------------------------------------------
// Count the total number of data elements. This
// value should always be an even number.

   $TotalDataCount = count($XY);

// ------------------------------------------------------------
// Number of data pairs.  This value should be an integer value
// exactly equal to 1/2 the total number of data points. If not,
// then there is an odd mismatched data point.

   $n = $TotalDataCount / 2;

// -----------------------------------------------------------
// Return error message if data vector element count mismatch.
// For every X value there must be a corresponding Y value or
// an XY data count mismatch error occurs.  An error will also
// occur if insufficient data.  There must be at least two XY
// data points given.

   if ($TotalDataCount < 4 )
      {return "ERROR: There must be at least two XY data pairs.";}

   if ($n != floor($n + 0.5))
      {return "ERROR: XY Data Count Mismatch. Odd data element.";}

// ------------------------------------------------------------
// Compute number of XY data pairs.  This value is exactly half
// the value of the total number of data elements.

   $n = $TotalDataCount / 2;

// -------------------------------------------------------
// Construct separate XY data strings from the array data.
// The XY data strings should each contain the same number
// of data elements.

   for($i=0;   $i < $TotalDataCount;   $i+=2)
      {
       $XDataStr .= $XY[$i]   . " ";
       $YDataStr .= $XY[$i+1] . " ";
      }

// --------------------------------------------------------------
// Split the created XY data vector strings into matching indexed
// arrays.  For every X value there must be a matching Y value
// and no two X values can be identical.

   $X = preg_split("[ ]", trim($XDataStr));
   $Y = preg_split("[ ]", trim($YDataStr));

// ----------------------------------------
// Read X argument for which to interpolate
// the Y value from the given XY data.

   $x = trim($xArg);  if ($x == "") {$x = "0";}

// -----------------------------------
// Initialize Y summation accumulator.
   $y = 0.0;

// -----------------------------------------------------
// Compute Lagrangian product (Li) for given X argument.

   for ($i=0;   $i < $n;   $i++)
       {
        $Li = 1.0;

        for ($j=0;   $j < $n;   $j++)
            {
             if ($j != $i) // Skip this cycle when j == i
                {
                 $Li = ($Li * ($x - $X[$j])) / ($X[$i] - $X[$j]);
                }
            } // Next j

//      --------------------------------------
//      Accumulate sum of Yi polynomial terms.

        $y += ($Y[$i] * $Li);

       } // Next i

   return $y;

} // End of  LaGrange_Interp(...)




/*
   ========================================================================
   This function simply swaps all matching XY data pairs held in a
   space delimited string argument. It was designed as a companion
   to the Lagrange_Interp() function to easily facilitate the easy
   swapping of the XY data computation roles.

   The data trend can be linear or non-linear, however
   only 2 data pairs are needed for any linear trend.

   ERRORS:
   On error FALSE is returned.  An error occurs if
   odd number of data elements or < 4 elements.
   ========================================================================
*/

   function Swap_XY_Data_Columns ($XYDataStr)
{

// -------------------------------------------------------------------------
// Read space-delimited data string and split items into a sequential array.
   $XYArray = preg_split("[ ]", preg_replace("/\s+/", " ", trim($XYDataStr)));

   $XYArrayCount = count($XYArray);

   if ($XYArrayCount % 2 != 0 or $XYArrayCount < 4) {return FALSE;}

   $XYPairsCount = $XYArrayCount / 2;

// -------------------------------------------------------
// Construct and return output string of swapped XY pairs.
   $YXDataPairsStr = "";

   for ($i=1;   $i < $XYArrayCount;   $i+=2)
       {
        $YXDataPairsStr .= $XYArray[$i] . " " . $XYArray[$i-1] . "\n";
       }

   return trim($YXDataPairsStr);
}








/*
  ----------------------------------------------------------------------
  This function returns the time of an extremum (minimum or maximum), if
  any, within a given time vs event table.  For example, this function
  can be used to find the times of perihelion, aphelion, perigee, apogee
  or any general periapsis or apoapsis times.  It is based on a 5-point
  data table and the extremum is computed from a polynomial derived from
  the given data.

  ERRORS:
  No error checking is done and the function assumes that an extremum
  exists within the given data table.

  Possible Uses:
  XY-Data = JD vs DistKm

  ----------------------------------------------------------------------
*/


   function Extremum_5 ($DataTableStr)
{

// ---------------------------------
// Read data table and parse values.

   $DataTable = preg_replace("/\s+/", ' ', trim($DataTableStr));

   list ($x1,$y1, $x2,$y2, $x3,$y3, $x4,$y4, $x5,$y5)
   = preg_split("[ ]", $DataTable);

   $interval = $x2 - $x1;

   $a = $y2 - $y1;
   $b = $y3 - $y2;
   $c = $y4 - $y3;
   $d = $y5 - $y4;

   $e = $b - $a;
   $f = $c - $b;
   $g = $d - $c;
   $h = $f - $e;
   $i = $g - $f;
   $j = $i - $h;

   $k = $j/24;
   $m = ($h + $i)/12;
   $n = $f/2 - $k;
   $p = ($b + $c)/2 - $m;

   $q = $r = 0;

   while ($r < 25)
  {
   $s = 6*($b + $c) - $h - $i + 3*$q*$q*($h + $i) + 2*$q*$q*$q*$j;
   $t = $j - 12*$f;
   $q = $s/$t;

   $r++;
  }
   return $q * $interval + $x3;

} // End of  Extremum5(...)



/*
   INTERPOLATES THE TIME IN HOURS GIVEN AN INTERPOLATION TABLE
   AND JDTT ARGUMENT. FOR ASTRONOMY, THIS CAN BE USED TO INTER-
   POLATE THE R.A. FOR A SPECIFIED JDTT ARGUMENT.
*/
   function Interp_Hours ($HoursInterpTableStr, $JDTTArg)
{
   $wArray = preg_split("[\n]", trim($HoursInterpTableStr));
   $wCount = count($wArray);

   $JDArg = floatval($JDTTArg);

   for ($i=1;   $i < $wCount;   $i++)
  {
   $CurrLine = strip_wspace($wArray[$i]);
   $PrevLine = strip_wspace($wArray[$i-1]);

   list($CurrJDTT, $CurrRAHr) = preg_split("[ ]", $CurrLine);
   list($PrevJDTT, $PrevRAHr) = preg_split("[ ]", $PrevLine);

   $diff = $CurrRAHr - $PrevRAHr;

   if ($diff < 0) {$CurrRAHr += 24;}

   $wArray[$i] = "$CurrJDTT $CurrRAHr\n";

  }

// CONSTRUCT FINAL INTERPOLATION TABLE READY FOR USE.
   $w = '';
   for ($i=0;   $i < $wCount;   $i++)
  {
   $w .= trim($wArray[$i]) . "\n";
  }
   $HoursInterpTableStr = strip_wspace($w);


// PERFORM INTERPOLATION FOR hours VALUE CORRESPONDING TO JDTT VALUE.
   $hours = Lagrange_interp($HoursInterpTableStr, $JDArg);

// IF hours > 24, THEN SUBTRACT 24.
   $hours -= ($hours > 24)? 24:0;

   return sprintf("%2.7f", $hours);

} // End of  Interp_Hours(...)





/*
   This function computes the standard deviation of a set of
   comma, semicolon or space-delimited numbers.

   $NumericDataString = String of delimited numeric values.
                        Example: "123.2  121.5  121.1  124.7"

   SampleMode = 0 or 1 = default.
   0 = Population sample randomly drawn from larger population
   1 = Sample is the total population. (= default)
*/

   function Std_Deviation ($NumericDataString, $SampleMode=1)
{
   $w = str_replace(",", " ", $NumericDataString);
   $w = str_replace(";", " ", $w);
   $w = preg_replace('/\s+/', ' ', trim($w));
   $WorkString = $w;

   $SampleMode = (abs(intval($SampleMode)) != 0)? 1:0;

   $N = $sum = 0;

   while ($WorkString != '')
  {
   $N++;

   $i = strpos($WorkString, ' ');

   if ($i === FALSE)
      {
       $sum  = $sum + $WorkString;
       $mean = $sum / $N;
       break;
      }

   $xi = trim(substr($WorkString, 0, $i));
   $sum += $xi;
   $mean = $sum / $N;

   $WorkString = trim(substr($WorkString, $i, strlen($WorkString)));
  }

   $N = $sum = 0;

   while ($w != '')
  {
   $N++;

   $i = strpos($w, ' ');

   if ($i === FALSE)
      {
       $diff = $w - $mean;
       $sum += ($diff * $diff);
       break;
      }

   $xi = trim(substr($w, 0, $i));
   $diff = $xi - $mean;
   $sum += ($diff * $diff);
   $w = trim(substr($w, $i, strlen($w)));
  }
   $variance = $sum / ($N - $SampleMode);

   return sqrt($variance);

} // End of  Std_Deviation(...)







/*
 ------------------------------------------------------------------------------
 This function computes the standard statistics for a numerical data string.
 The returned string contains the following computed statistics:
 n                  = Number of data elements (numbers to be processed)
 $sum               = Sum of all the data
 $mean              = Arithmetic mean of the data
 $SUMsquaresOfDiffs = Sum of the squares of the differences
 $variance0         = Population variance
 $variance1         = Sample variance
 $StdDev0           = Population standard deviation
 $StdDev1           = Sample standard deviation";
*/

   function Std_Dev_Stats ($NumericDataString)
{
   $w = str_replace(',',  ' ', $NumericDataString);
   $w = str_replace(';',  ' ', $w);
   $w = str_replace('\n', ' ', $w);
   $WorkString = preg_replace('/\s+/', ' ', trim($w));

   $n = $SUMdata = 0;

   while ($WorkString != '')
  {
   $n++;

   $i = strpos($WorkString, ' ');

   if ($i === FALSE)
      {
       $SUMdata += $WorkString;
       $mean = $SUMdata / $n;
       break;
      }

   $xi = trim(substr($WorkString, 0, $i));
   $SUMdata += $xi;
   $mean = $SUMdata / $n;

   $WorkString = trim(substr($WorkString, $i, strlen($WorkString)));
  }

// *************************************************************************
// Cycle through data computing the sum of squares of differences from mean.
// The mean of this summation of squares is the population variance.

   $w = str_replace(",", " ", $NumericDataString);
   $w = str_replace(";", " ", $w);
   $WorkString = preg_replace('/\s+/', ' ', trim($w));

   $n = $SUMsquaresOfDiffs = 0;

   while ($WorkString != '')
  {
   $n++;

   $i = strpos($WorkString, ' ');

   if ($i === FALSE)
      {
       $diff = $WorkString - $mean;
       $SUMsquaresOfDiffs += ($diff * $diff);
       break;
      }

   $xi = trim(substr($WorkString, 0, $i));
   $diff = $xi - $mean;
   $SUMsquaresOfDiffs += ($diff * $diff);
   $WorkString = trim(substr($WorkString, $i, strlen($WorkString)));
  }

   $variance0 = $SUMsquaresOfDiffs / ($n-0); // sigma
   $variance1 = $SUMsquaresOfDiffs / ($n-1); // s

   $StdDev0 = sqrt($variance0); // SqRt(sigma)
   $StdDev1 = sqrt($variance1); // SqRt(s)

   $StdDevStats = "$n $SUMdata $mean $SUMsquaresOfDiffs $variance0 $variance1 $StdDev0 $StdDev1";

   return $StdDevStats;

} // End of  Std_Dev_Stats(...)



/*
   ========================================================================
   Returns the compass symbol corresponding to azimuth angle in degrees and
   zero-mode.  The default zero azimuth direction is north.

   ZeroModeSymbol = 'N' (North = 0 convention = Default)
   ZeroModeSymbol = 'S' (South = 0 convention = Optional)

   ========================================================================
*/

   function Compass_Symbol ($AzimDeg, $ZeroModeSymbol='N')
{
   $a = floatval($AzimDeg) + ((strtoupper($ZeroModeSymbol) == 'S')? 180:0);

   $a -= 360*floor($a/360);

   $i = ($a < 11.25 or $a > 348.75)? 0 : floor(0.5 + $a/22.5);

   if ($ZeroModeSymbol != 'S')
   {return trim(substr("N  NNENE ENEE  ESESE SSES  SSWSW WSWW  WNWNW NNWN", 3*$i, 3));}
   else
   {return trim(substr("S  SSWSW WSWW  WNWNW NNWN  NNENE ENEE  ESESE SSES", 3*$i, 3));}

} // End of  Compass_Symbol (...)



/*
  ========================================================================
  Computes the theoretical astronomical Delta-T estimate in seconds for
  any dates between 1600 and 2200, based on the NASA polymomial
  expressions for Delta T.

  NASA Ref:
  http://eclipse.gsfc.nasa.gov/LEcat5/deltatpoly.html


  ARGUMENTS:
  Y = Year number (1600 to 2200 AD)
  mNum = Month number(01 to 12)
  EXAMPLE: Ym = 195005 (= 1950 May)

  ERRORS:
  FALSE is returned on error.  An error results if either
  the year or month number is invalid.  A zero year value
  will return an error.
  ========================================================================
*/

   function Delta_T ($yyyymnOrJDTT, $RetMode=' ')
{
// ERROR if date value is non-numeric.
// It must be a positive 6-digit integer value.
// yyyymn = 100*Y + m
//
// Modifications.
// Can also use a 7-digit JD number.
// A JD number will be used to determine the year and month values.
//
// Now has a DMS return mode to save work.
// HMS = Return XXh XXm XXs
//  :  = Return hh:mm:ss

// --------------------------------------------
// Read return mode.  Empty = Default = Seconds
// This is CASE SENSITIVE
//  H = Return XXh XXm XXs
//  h = H.hhh
//  : = Return hh:mm:ss

   $RMode = substr(trim($RetMode), 0,1);  if ($RMode == '') {$RMode = ' ';}

   $dTSeconds = _Delta_T ($yyyymnOrJDTT);

   $DeltaT = sprintf("%+3.0f", round($dTSeconds, 0));

   if ($DeltaT === FALSE){return FALSE;;}

   if ($RMode == ':')
  {
   $dTHrs = $dTSeconds/3600;
   $DeltaT = Hours_to_HMS($dTHrs, 0, '+', ':');
  }

   if ($RMode == 'h')
  {
   $DeltaT = sprintf("%+1.9f", $dTSeconds/3600);
  }

   if ($RMode == 'H')
  {
   $dTHrs = $dTSeconds/3600;
   $DeltaT = Hours_to_HMS($dTHrs, 0, '+', $RMode);
  }

// Return Delta T in indicated mode.
   return $DeltaT;

} // End of  Delta_T(...)

/*
   This is a subroutine for the upper-level
   Delta T function above that calls it.
*/
   FUNCTION _Delta_T ($yyyymnOrJDTT)
{
// ERROR if date value is non-numeric.
// It must be a positive 6-digit integer value.
// yyyymn = 100*Y + m
//
// Modifications.
// Can also use a 7-digit JD number.
// A JD number will be used to determine the year and month values.
//
// Now has a DMS return mode to save work.
// HMS = Return XXh XXm XXs
//  :  = Return hh:mm:ss

// Read input argument.
   $w = abs(floatval(trim($yyyymnOrJDTT)));

// ERROR returns FALSE if argument is invalid value.
   if ($w == 0) {return FALSE;}

// If argument contains a decimal point, then it is assumed to be a
// Julian date value and is converted into the corresponding JD number.
   if (strpos($w, '.') !== FALSE) {$w = floor($w + 0.5);}

// AT THIS POINT, w SHOULD BE EITHER A 6 OR 7-DIGIT POSITIVE INTEGER.
//
// 6 digits = Integer-encoded date elements yyyymn = 100*yyyy + mn
// 7 digits = JD number corresponding to Julian Date argument value.

// ERROR returns FALSE if either argument is neither 6 or 7 digits.
   if (strlen($w) < 6 or strlen($w) > 7) {return FALSE;}


// HANDLE CASE OF JULIAN DATE OR JD NUMBER ARGUMENT.
// If 7-digit integer, then compute corresponding
// mn, dn, yyyy values. The returned dn is not used.
   if (strlen($w) == 7)
  {
   list($mn, $dn, $yyyy) = preg_split("[\/]", JDtoGregorian($w));

   $w = 100*$yyyy + $mn;
  }

// Otherwise, it is assumed that the argument is a 6-digit yyyymn value.
   $yyyymn = $w;

// Parse yyyymn value.

   $Y = substr($yyyymn, 0,4);
   $mNum = intval(substr($yyyymn, 4,2));

// ERROR if year value is invalid. It must
// be in the range from 1600 to 2200.

   if ($Y < 1600 or $Y == 0 or $Y > 2200) {return FALSE;}

// ERROR if month value is outside the range from 1 to 12.

   if ($mNum < 1 or $mNum > 12) {return FALSE;}

// If a negative (BC) year is given , it is converted into the
// corresponding mathematical year by adding 1.
//
// The year 100 BC (-100), becomes the mathematical year -99.
// This will never be the case here because this implementation
// of the polynomials is not meant to cover ancient dates that
// use negative year values, only years from 1600 AD onward.

   $y = $Y + (($Y < 0)? 1:0) + ($mNum - 0.5)/12;


   if ($Y >= 1600 and $Y < 1700)
      {
       $u  = $y - 1600;

       return 120 - 0.9808*$u - 0.01532*$u*$u + $u*$u*$u/7129;
      }


   if ($Y >= 1700 and $Y < 1800)
      {
       $u  = $y - 1700;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;

       return 8.83 + 0.1603*$u
                   - 0.0059285*$u2
                   + 0.00013336*$u3
                   - $u4/1174000;
      }


   if ($Y >= 1800 and $Y < 1860)
      {
       $u  = $y - 1800;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;
       $u6 = $u * $u5;
       $u7 = $u * $u6;

       return 13.72 - 0.332447*$u
                    + 0.0068612*$u2
                    + 0.0041116*$u3
                    - 0.00037436*$u4
                    + 0.0000121272*$u5
                    - 0.0000001699*$u6
                    + 0.000000000875*$u7;
      }


   if ($Y >= 1860 and $Y < 1900)
      {
       $u  = $y - 1860;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;

       return 7.62 + 0.5737*$u
                   - 0.251754*$u2
                   + 0.01680668*$u3
                   - 0.0004473624*$u4
                   + $u5 / 233174;
      }


   if ($Y >= 1900 and $Y < 1920)
      {
       $u  = $y - 1900;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;

       return 1.494119*$u - 2.79
                          - 0.0598939*$u2
                          + 0.0061966*$u3
                          - 0.000197*$u4;
      }


   if ($Y >= 1920 and $Y < 1941)
      {
       $u  = $y - 1920;
       $u2 = $u * $u;
       $u3 = $u * $u2;

       return 21.20 + 0.84493*$u - 0.076100*$u2 + 0.0020936*$u3;
      }


   if ($Y >= 1941 and $Y < 1961)
      {
       $u  = $y - 1950;
       $u2 = $u * $u;
       $u3 = $u * $u2;

       return 29.07 + 0.407*$u - $u2/233 + $u3/2547;
      }


   if ($Y >= 1961 and $Y < 1986)
      {
       $u = $y - 1975;
       $u2 = $u * $u;
       $u3 = $u * $u2;

       return 45.45 + 1.067*$u - $u2/260 - $u3/718;
      }


   if ($Y >= 1986 and $Y < 2005)
      {
       $u  = $y - 2000;
       $u2 = $u * $u;
       $u3 = $u * $u2;
       $u4 = $u * $u3;
       $u5 = $u * $u4;

       return 63.86 + 0.3345*$u
                    - 0.060374*$u2
                    + 0.0017275*$u3
                    + 0.000651814*$u4
                    + 0.00002373599*$u5;
      }


   if ($Y >= 2005 and $Y < 2050)
      {
       $u  = $y - 2000;

       return 62.92 + 0.32217*$u + 0.005589*$u*$u;
      }


   if ($Y >= 2050 and $Y <= 2150)
      {
       $u  = ($y - 1820) / 100;

       return 32*$u*$u - 0.5628*(2150 - $y) - 20;
      }


   if ($Y > 2150 and $Y <= 3000)
      {
       $u  = ($y - 1820) / 100;

       return 32*$u*$u - 20;
      }

   return FALSE;

} // End of  _Delta_T(...)







/*
  ========================================================================
  Computes the nutation in obliquity (delta epsilon) in
  degrees according to the IAU 2000B nutation series.

  This value is applied to the mean obliquity of the ecliptic to
  obtain the true, apparent obliquity.

  The series consists of 77 terms.

  ARGUMENT:
  $JD = JD number at the moment (TDB) for which the
        nutation in obliquity is required.

  ERRORS:
  No error checking.

  ========================================================================
*/


// THIS WILL BE NEW FUTURE VERSION TO REPLACE
// THE DEPRECATED dEps() FUNCTION VERSION.
// BOTH ARE FUNCTIONALLY IDENTICAL.
   function Delta_Eps ($JDTT)
  {
   return dEps ($JDTT);
  }

   FUNCTION dEps ($JDTT)
{
   $T = ($JDTT - 2451545) / 36525;

   $T2 = $T * $T;
   $T3 = $T * $T2;
   $T4 = $T * $T3;

// -------------------------------------------
// Compute mean anomaly of the Moon in radians

   $L  = deg2rad((485868.249036 + 1717915923.2178*$T + 31.8792*$T2
       + 0.051635*$T3 - 0.0002447*$T4) / 3600);


// ------------------------------------------
// Compute mean anomaly of the Sun in radians

   $Lp = deg2rad((1287104.79305 + 129596581.0481*$T
       - 0.5532*$T2  + 0.000136*$T3 - 0.00001149*$T4) / 3600);


// ------------------------------------------------------------
// Compute mean argument of the latitude of the Moon in radians

   $F  = deg2rad((335779.526232 + 1739527262.8478*$T
       - 12.7512*$T2 - 0.001037*$T3 + 0.00000417*$T4) / 3600);


// -----------------------------------------------------------
// Compute mean elongation of the Moon from the Sun in radians

   $D  = deg2rad((1072260.70369 + 1602961601.2090*$T
       - 6.3706*$T2  + 0.006593*$T3 - 0.00003169*$T4) / 3600);


// ------------------------------------------------------------------------
// Compute mean longitude of the mean ascending node of the Moon in radians

   $Om = deg2rad((450160.398036 - 6962890.5431*$T
       + 7.4722*$T2  + 0.007702*$T3 - 0.00005939*$T4) / 3600);

// --------------------------------------------------------------
// Sum series for nutation in obliquity (dEps) in arc sec * 10E+7

$s = 0;
$s += (92052331 + 9086*$T)*cos($Om) + 15377*sin($Om);
$s += (5730336 - 3015*$T)*cos(2*($F - $D + $Om)) - 4587*sin(2*($F - $D + $Om));
$s += (978459 - 485*$T)*cos(2*($F + $Om)) + 1374*sin(2*($F + $Om));
$s += (-897492 + 470*$T)*cos(2*$Om) - 291*sin(2*$Om);
$s += (73871 - 184*$T)*cos($Lp) - 1924*sin($Lp);
$s += (224386 - 677*$T)*cos($Lp + 2*($F - $D + $Om)) - 174*sin($Lp + 2*($F - $D + $Om));
$s -= 6750*cos($L) - 358*sin($L);
$s += (200728 + 18*$T)*cos(2*$F + $Om) + 318*sin(2*$F + $Om);
$s += (129025 - 63*$T)*cos($L + 2*($F + $Om)) + 367*sin($L + 2*($F + $Om));
$s += (-95929 + 299*$T)*cos(2*($F - $D + $Om) - $Lp) + 132*sin(2*($F - $D + $Om) - $Lp);
$s += (-68982 - 9*$T)*cos(2*($F - $D) + $Om) + 39*sin(2*($F - $D) + $Om);
$s += (-53311 + 32*$T)*cos(2*($F + $Om) - $L) - 4*sin(2*($F + $Om) - $L);
$s -= 1235*cos(2*$D - $L) - 82*sin(2*$D - $L);
$s -= 33228*cos($L + $Om) + 9*sin($L + $Om);
$s += 31429*cos($Om - $L) - 75*sin($Om - $L);
$s += (25543 - 11*$T)*cos(2*($F + $D + $Om) - $L) + 66*sin(2*($F + $D + $Om) - $L);
$s += 26366*cos($L + 2*$F + $Om) + 78*sin($L + 2*$F + $Om);
$s += (-24236 - 10*$T)*cos(2*($F - $L) + $Om) + 20*sin(2*($F - $L) + $Om);
$s -= 1220*cos(2*$D) - 29*sin(2*$D);
$s += (16452 - 11*$T)*cos(2*($F + $D + $Om)) + 68*sin(2*($F + $D + $Om));
$s -= 13870*cos(-2*($Lp - $F + $D - $Om));
$s += 477*cos(2*($D - $L)) - 25*sin(2*($D - $L));
$s += (13238 - 11*$T)*cos(2*($L + $F + $Om)) + 59*sin(2*($L + $F + $Om));
$s += (-12338 + 10*$T)*cos($L + 2*($F - $D + $Om)) - 3*sin($L + 2*($F - $D + $Om));
$s -= 10758*cos(2*$F + $Om - $L) + 3*sin(2*$F + $Om - $L);
$s -= 609*cos(2*$L) - 13*sin(2*$L);
$s -= 550*cos(2*$F) - 11*sin(2*$F);
$s += (8551 - 2*$T)*cos($Lp + $Om) - 45*sin($Lp + $Om);
$s -= 8001*cos(-$L + 2*$D + $Om) + sin(-$L + 2*$D + $Om);
$s += (6850 - 42*$T)*cos(2*($Lp + $F - $D + $Om)) - 5*sin(2*($Lp + $F - $D + $Om));
$s -= 167*cos(2*($D - $F)) - 13*sin(2*($D - $F));
$s += 6953*cos($L - 2*$D + $Om) - 14*sin($L - 2*$D + $Om);
$s += 6415*cos($Om - $Lp) + 26*sin($Om - $Lp);
$s += 5222*cos(2*($F + $D) + $Om - $L) + 15*sin(2*($F + $D) + $Om - $L);
$s += (168 - $T)*cos(2*$Lp) + 10*sin(2*$Lp);
$s += 3268*cos($L + 2*($F + $D + $Om)) + 19*sin($L + 2*($F + $D + $Om));
$s += 104*cos(2*($F - $L)) + 2*sin(2*($F - $L));
$s -= 3250*cos($Lp + 2*($F + $Om)) + 5*sin($Lp + 2*($F + $Om));
$s += 3353*cos(2*($F + $D) + $Om) + 14*sin(2*($F + $D) + $Om);
$s += 3070*cos(2*($F + $Om) - $Lp) + 4*sin(2*($F + $Om) - $Lp);
$s += 3272*cos(2*$D + $Om) + 4*sin(2*$D + $Om);
$s -= 3045*cos($L + 2*($F - $D) + $Om) + sin($L + 2*($F - $D) + $Om);
$s -= 2768*cos(2*($L + $F - $D + $Om)) + 4*sin(2*($L + $F - $D + $Om));
$s += 3041*cos(2*($D - $L) + $Om) - 5*sin(2*($D - $L) + $Om);
$s += 2695*cos(2*($L + $F) + $Om) + 12*sin(2*($L + $F) + $Om);
$s += 2719*cos(2*($F - $D) + $Om - $Lp) - 3*sin(2*($F - $D) + $Om - $Lp);
$s += 2720*cos($Om - 2*$D) - 9*sin($Om - 2*$D);
$s -= 51*cos(-$L - $Lp + 2*$D) - 4*sin(-$L - $Lp + 2*$D);
$s -= 2206*cos(2*($L - $D) + $Om) - sin(2*($L - $D) + $Om);
$s -= 199*cos($L + 2*$D) - 2*sin($L + 2*$D);
$s -= 1900*cos($Lp + 2*($F - $D) + $Om) - sin($Lp + 2*($F - $D) + $Om);
$s -= 41*cos($L - $Lp) - 3*sin($L - $Lp);
$s += 1313*cos(-2*($L - $F - $Om)) - sin(-2*($L - $F - $Om));
$s += 1233*cos(3*$L + 2*($F + $Om)) + 7*sin(3*$L + 2*($F + $Om));
$s -= 81*cos(-$Lp + 2*$D) - 2*sin(-$Lp + 2*$D);
$s += 1232*cos($L - $Lp + 2*($F + $Om)) + 4*sin($L - $Lp + 2*($F + $Om));
$s -= 20*cos($D) + 2*sin($D);
$s += 1207*cos(2*($F + $D + $Om) - $L - $Lp) + 3*sin(2*($F + $D + $Om) - $L - $Lp);
$s += 40*cos(2*$F - $L) - 2*sin(2*$F - $L);
$s += 1129*cos(-$Lp + 2*($F + $D + $Om)) + 5*sin(-$Lp + 2*($F + $D + $Om));
$s += 1266*cos($Om - 2*$L) - 4*sin($Om - 2*$L);
$s -= 1062*cos($L + $Lp + 2*($F + $Om)) + 3*sin($L + $Lp + 2*($F + $Om));
$s -= 1129*cos(2*$L + $Om) + 2*sin(2*$L + $Om);
$s -= 9*cos($Lp + $D - $L);
$s += 35*cos($L + $Lp) - 2*sin($L + $Lp);
$s -= 107*cos($L + 2*$F) - sin($L + 2*$F);
$s += 1073*cos(2*($F - $D) + $Om - $L) - 2*sin(2*($F - $D) + $Om - $L);
$s += 854*cos($L + 2*$Om);
$s -= 553*cos($D - $L) + 139*sin($D - $L);
$s -= 710*cos(2*($F + $Om) + $D) + 2*sin(2*($F + $Om) + $D);
$s += 647*cos(2*($F + 2*$D + $Om) - $L) + 4*sin(2*($F + 2*$D + $Om) - $L);
$s -= 700*cos($Lp + $D + $Om - $L);
$s += 672*cos(-2*($Lp - $F + $D) + $Om);
$s += 663*cos($L + 2*($F + $D) + $Om) + 4*sin($L + 2*($F + $D) + $Om);
$s -= 594*cos(-2*($L - $F - $D - $Om)) + 2*sin(-2*($L - $F - $D - $Om));
$s -= 610*cos(2*$Om - $L) - 2*sin(2*$Om - $L);
$s -= 556*cos($L + $Lp + 2*($F - $D + $Om));

// ----------------------------
// Return nutation in obliquity
// expressed in degrees.

   return $s / 36000000000;


} // End of  dEps(...)



/*

   ========================================================================
   Computes the nutation in ecliptical longitude (Delta psi)
   in degrees according to the IAU 2000B nutation series.

   This value is applied to the mean ecliptical longitude to obtain the
   true, apparent ecliptical longitude.

   The series consists of 77 terms.

   ARGUMENT:
   $JDTT = Astronomical JD number corresponding to date and time (TT).

   ERRORS:
   No error checking.
   ========================================================================
*/


   function Delta_Psi($JDTT)
  {
   return dPsi ($JDTT);
  }

   function dPsi ($JDTT)
{
// ---------------------------------------------
// Compute time (T) in Julian centuries reckoned
// from J2000.0 and 4 powers of T.

   $T  = ($JDTT - 2451545) / 36525;

   $T2 = $T * $T;
   $T3 = $T * $T2;
   $T4 = $T * $T3;

// -------------------------------------------
// Compute mean anomaly of the Moon in radians

   $L  = deg2rad((485868.249036 + 1717915923.2178*$T + 31.8792*$T2
       + 0.051635*$T3 - 0.0002447*$T4) / 3600);


// ------------------------------------------
// Compute mean anomaly of the Sun in radians

   $Lp = deg2rad((1287104.79305 + 129596581.0481*$T
       - 0.5532*$T2  + 0.000136*$T3 - 0.00001149*$T4) / 3600);


// ------------------------------------------------------------
// Compute mean argument of the latitude of the Moon in radians

   $F  = deg2rad((335779.526232 + 1739527262.8478*$T
       - 12.7512*$T2 - 0.001037*$T3 + 0.00000417*$T4) / 3600);


// -----------------------------------------------------------
// Compute mean elongation of the Moon from the Sun in radians

   $D  = deg2rad((1072260.70369 + 1602961601.2090*$T
       - 6.3706*$T2  + 0.006593*$T3 - 0.00003169*$T4) / 3600);


// ------------------------------------------------------------------------
// Compute mean longitude of the mean ascending node of the Moon in radians

   $Om = deg2rad((450160.398036 - 6962890.5431*$T
       + 7.4722*$T2  + 0.007702*$T3 - 0.00005939*$T4) / 3600);

// -----------------------------------------------------------------------
// Sum 2000B series for nutation in longitude (dPsi) in arc sec * 10000000

$s = 0;
$s += (-172064161 - 174666*$T)*sin($Om) + 33386*cos($Om);
$s += (-13170906 - 1675*$T)*sin(2*($F - $D + $Om)) - 13696*cos(2*($F - $D + $Om));
$s += (-2276413 - 234*$T)*sin(2*($F + $Om)) + 2796*cos(2*($F + $Om));
$s += (2074554 + 207*$T)*sin(2*$Om) - 698*cos(2*$Om);
$s += (1475877 - 3633*$T)*sin($Lp) + 11817*cos($Lp);
$s += (-516821 + 1226*$T)*sin($Lp + 2*($F - $D + $Om)) - 524*cos($Lp + 2*($F - $D + $Om));
$s += (711159 + 73*$T)*sin($L) - 872*cos($L);
$s += (-387298 - 367*$T)*sin(2*$F + $Om) + 380*cos(2*$F + $Om);
$s += (-301461 - 36*$T)*sin($L + 2*($F + $Om)) + 816*cos($L + 2*($F + $Om));
$s += (215829 - 494*$T)*sin(2*($F - $D + $Om) - $Lp) + 111*cos(2*($F - $D + $Om) - $Lp);
$s += (128227 + 137*$T)*sin(2*($F - $D) + $Om) + 181*cos(2*($F - $D) + $Om);
$s += (123457 + 11*$T)*sin(2*($F + $Om) - $L) + 19*cos(2*($F + $Om) - $L);
$s += (156994 + 10*$T)*sin(2*$D - $L) - 168*cos(2*$D - $L);
$s += (63110 + 63*$T)*sin($L + $Om) + 27*cos($L + $Om);
$s += (-57976 - 63*$T)*sin($Om - $L) - 189*cos($Om - $L);
$s += (-59641 - 11*$T)*sin(2*($F + $D + $Om) - $L) + 149*cos(2*($F + $D + $Om) - $L);
$s += (-51613 - 42*$T)*sin($L + 2*$F + $Om) + 129*cos($L + 2*$F + $Om);
$s += (45893 + 50*$T)*sin(2*($F - $L) + $Om) + 31*cos(2*($F - $L) + $Om);
$s += (63384 + 11*$T)*sin(2*$D) - 150*cos(2*$D);
$s += (-38571 - $T)*sin(2*($F + $D + $Om)) + 158*cos(2*($F + $D + $Om));
$s += 32481*sin(2*($F - $Lp - $D + $Om));
$s -= 47722*sin(2*($D - $L)) + 18*cos(2*($D - $L));
$s += (-31046 - $T)*sin(2*($L + $F + $Om)) + 131*cos(2*($L + $F + $Om));
$s += 28593*sin($L + 2*($F - $D + $Om)) - cos($L + 2*($F - $D + $Om));
$s += (20441 + 21*$T)*sin(2*$F - $L + $Om) + 10*cos(2*$F - $L + $Om);
$s += 29243*sin(2*$L) - 74*cos(2*$L);
$s += 25887*sin(2*$F) - 66*cos(2*$F);
$s += (-14053 - 25*$T)*sin($Lp + $Om) + 79*cos($Lp + $Om);
$s += (15164 + 10*$T)*sin(2*$D - $L + $Om) + 11*cos(2*$D - $L + $Om);
$s += (-15794 + 72*$T)*sin(2*($Lp + $F - $D + $Om)) - 16*cos(2*($Lp + $F - $D + $Om));
$s += 21783*sin(2*($D - $F)) + 13*cos(2*($D - $F));
$s += (-12873 - 10*$T)*sin($L - 2*$D + $Om) - 37*cos($L - 2*$D + $Om);
$s += (-12654 + 11*$T)*sin($Om - $Lp) + 63*cos($Om - $Lp);
$s -= 10204*sin(2*($F + $D) - $L + $Om) - 25*cos(2*($F + $D) - $L + $Om);
$s += (16707 - 85*$T)*sin(2*$Lp) - 10*cos(2*$Lp);
$s -= 7691*sin($L + 2*($F + $D + $Om)) - 44*cos($L + 2*($F + $D + $Om));
$s -= 11024*sin(-2*$L + 2*$F) + 14*cos(2*($F - $L));
$s += (7566 - 21*$T)*sin($Lp + 2*($F + $Om)) - 11*cos($Lp + 2*($F + $Om));
$s += (-6637 - 11*$T)*sin(2*($F + $D) + $Om) + 25*cos(2*($F + $D) + $Om);
$s += (-7141 + 21*$T)*sin(2*($F + $Om) - $Lp) + 8*cos(2*($F + $Om) - $Lp);
$s += (-6302 - 11*$T)*sin(2*$D + $Om) + 2*cos(2*$D + $Om);
$s += (5800 + 10*$T)*sin($L + 2*($F - $D) + $Om) + 2*cos($L + 2*($F - $D) + $Om);
$s += 6443*sin(2*($L + $F - $D + $Om)) - 7*cos(2*($L + $F - $D + $Om));
$s += (-5774 - 11*$T)*sin(2*($D - $L) + $Om) - 15*cos(2*($D - $L) + $Om);
$s -= 5350*sin(2*($L + $F) + $Om) - 21*cos(2*($L + $F) + $Om);
$s += (-4752 - 11*$T)*sin(2*($F - $D) - $Lp + $Om) - 3*cos(2*($F - $D) - $Lp + $Om);
$s += (-4940 - 11*$T)*sin($Om - 2*$D) - 21*cos($Om - 2*$D);
$s += 7350*sin(2*$D - $L - $Lp) - 8*cos(2*$D - $L - $Lp);
$s += 4065*sin(2*($L - $D) + $Om) + 6*cos(2*($L - $D) + $Om);
$s += 6579*sin($L + 2*$D) - 24*cos($L + 2*$D);
$s += 3579*sin($Lp + 2*($F - $D) + $Om) + 5*cos($Lp + 2*($F - $D) + $Om);
$s += 4725*sin($L - $Lp) - 6*cos($L - $Lp);
$s -= 3075*sin(2*($F - $L  + $Om)) + 2*cos(2*($F - $L  + $Om));
$s -= 2904*sin(3*$L + 2*($F + $Om)) - 15*cos(3*$L + 2*($F + $Om));
$s += 4348*sin(2*$D - $Lp) - 10*cos(2*$D - $Lp);
$s -= 2878*sin($L - $Lp + 2*($F + $Om)) - 8*cos($L - $Lp + 2*($F + $Om));
$s -= 4230*sin($D) - 5*cos($D);
$s -= 2819*sin(2*($F + $D + $Om) - $L - $Lp) - 7*cos(2*($F + $D + $Om) - $L - $Lp);
$s -= 4056*sin(2*$F - $L) - 5*cos(2*$F - $L);
$s -= 2647*sin(2*($F + $D + $Om) - $Lp) - 11*cos(2*($F + $D + $Om) - $Lp);
$s -= 2294*sin($Om - 2*$L) + 10*cos($Om - 2*$L);
$s += 2481*sin($L + $Lp + 2*($F + $Om)) - 7*cos($L + $Lp + 2*($F + $Om));
$s += 2179*sin(2*$L + $Om) - 2*cos(2*$L + $Om);
$s += 3276*sin($Lp - $L + $D) + cos($Lp - $L + $D);
$s -= 3389*sin($L + $Lp) - 5*cos($L + $Lp);
$s += 3339*sin($L + 2*$F) - 13*cos($L + 2*$F);
$s -= 1987*sin(2*($F - $D) - $L + $Om) + 6*cos(2*($F - $D) - $L + $Om);
$s -= 1981*sin($L + 2*$Om);
$s += 4026*sin($D - $L) - 353*cos($D - $L);
$s += 1660*sin($D + 2*($F + $Om)) - 5*cos($D + 2*($F + $Om));
$s -= 1521*sin(2*($F + 2*$D + $Om) - $L) - 9*cos(2*($F + 2*$D + $Om) - $L);
$s += 1314*sin($Lp - $L + $D + $Om);
$s -= 1283*sin(2*($F - $Lp - $D) + $Om);
$s -= 1331*sin($L + 2*($F + $D) + $Om) - 8*cos($L + 2*($F + $D) + $Om);
$s += 1383*sin(2*($F - $L + $D + $Om)) - 2*cos(2*($F - $L + $D + $Om));
$s += 1405*sin(2*$Om - $L) + 4*cos(2*$Om - $L);
$s += 1290*sin($L + $Lp + 2*($F - $D + $Om));

// ----------------------------------------
// Return nutation in longitude in degrees.

   return $s / 36000000000;

} // End of  dPsi ()



/*
   Computes the nutation in right ascension or the equation of the
   equinoxes, in degrees.

   This value is the difference (True Equinox) minus (Mean Equinox).

   The IAU 2000B nutation series is applied.

   LEVEL 0 DEPENDENCIES:
   Mean_Eps()
   dEps()
   dPsi()


   NOTE:
   Degrees * 240 = Seconds of RA
   ========================================================================
*/

   function Delta_Equinox($JDTT)
  {
   return dEquinox ($JDTT);
  }

   FUNCTION dEquinox ($JDTT)
{
// ---------------------------------------------
// COMPUTE COSINE OF TRUE OBLIQUITY OF ECLIPTIC.

   $CosEpsTrue = cosd(Mean_Eps($JDTT) + Delta_Eps($JDTT));

// --------------------------------------------
// RETURN EQUATION OF THE EQUINOXES IN DEGREES.

   return Delta_Psi($JDTT) * $CosEpsTrue;

} // End of  dEquinox()



/*
   This function computes the geocentric ecliptical coordinates,
   given the JDTT and geocentric equatorial coordinates.

   INPUT:
   JDTT    = JDTT corresponding to Date/Time/Zone/dT
   RAHrs   = Geocentric equatorial RA of body in hours
   DeclDeg = Geocentric equatorial declination of body in degrees

   OUTPUT:
   Lm = Geocentric ecliptical longitude of body in degrees
   Bm = Geocentric ecliptical latitude of body in degrees

*/
   function Equ_to_Ecl_Coords($JDTT, $RAHrs, $DeclDeg)
{
// READ ARGUMENT VALUES.
   $J = $JDTT;
   $a = $RAHrs * 15; // Compute RA in degrees.
   $d = $DeclDeg;

// COMPUTE THE TRUE OBLIQUITY OF THE ECLIPTIC,
// CORRECTED FOR NUTATION.
   $e = Mean_Eps($J) + Delta_Eps($J);

// COMPUTE GEOCENTRIC ECLIPTICAL LONGITUDE.
   $Lm = atand2(sind($a)*cosd($e) + tand($d)*sind($e), cosd($a));

// COMPUTE GEOCENTRIC ECLIPTICAL LATITUDE.
   $Bm = asind(sind($d)*cosd($e) - cosd($d)*sind($e)*sind($a));

// DONE:
   return "$Lm $Bm";

} // END OF  Equ_to_Ecl_Coords(...)






/*
  ========================================================================
  Computes the mean obliquity of the ecliptic in degrees for the current
  JDTT argument corresponding to any given date/time.

  This ecliptic obliquity function is based on a formula derived by
  J. Laskar, Astronomy and Astrophysics, 1968, Vol. 157, p68.

  The estimated accuracy of this formula is 0.01 arc second between
  the years 1000 AD and 3000 AD and a few arc seconds after 10,000
  years.  It is only valid in the range 10,000 years either way
  of J2000.0

  Over the long term, it is more accurate than the formula adopted by
  the International Astronomical Union (IAU).


  ---------
  ARGUMENT:
  $JDTT = JD number at the moment (TDB) for which the mean obliquity
          of the ecliptic is required.
  ========================================================================
*/

   function Mean_Eps ($JDTT)
{

// ----------------------------------------------
// Compute the time in Julian centuries reckoned
// from J2000.0 corresponding to the JD argument
// and then divide by 100 for decamillennia.

   $t = ($JDTT - 2451545) / 36525 / 100;

// ------------------------------------------
// Compute mean obliquity (e) of the ecliptic
// expressed in arc seconds.

   $p = $t*$t;

   $e = 84381.448 - 4680.93*$t;

   $e -=    1.55*$p;  $p *= $t;
   $e += 1999.25*$p;  $p *= $t;
   $e -=   51.38*$p;  $p *= $t;
   $e -=  249.67*$p;  $p *= $t;
   $e -=   39.05*$p;  $p *= $t;
   $e +=    7.12*$p;  $p *= $t;
   $e +=   27.87*$p;  $p *= $t;
   $e +=    5.79*$p;  $p *= $t;
   $e +=    2.45*$p;

// -----------------------------------
// Done.  Return obliquity in degrees.
   return $e / 3600;

} // End of  Mean_Eps (...)






/*
   ===================================================================
   Computes the local or Greenwich mean sidereal time in degrees
   for any given JD and geographic longitude arguments.

   ARGUMENTS:
   JDTT    = JD argument for any given date and time (TT)

   LonDeg  = Geographic longitude. (Negative = West)
             Default longitude = 0 degrees (= Greenwich)

   ERRORS:
   FALSE is returned if either argument is invalid.
   ===================================================================
*/

   function Mean_Sid_Time ($JDTT, $LonDeg=0)
{
// ------------------------------
// Check for invalid argument(s).
   if ($JDTT < 2305447 or $JDTT > 2524959 or !is_numeric($LonDeg)) {return FALSE;}

// ------------------------------------------------
// Compute time (t) in Julian millennia as reckoned
// from J2000.0 and corresponding to the JD number.
   $t = ($JDTT - 2451545) / 36525;

// ------------------------------------------------
// Compute Greenwich mean sidereal time in degrees.
   $GSTMeanDeg = 280.46061837 + 360.98564736629*($JDTT - 2451545)
               + ($t*$t * 0.000387993) - ($t*$t*$t / 38710000);

// -----------------------------------
// Compute local mean sidereal time at
// the given geographic longitude.
   $LSTMeanDeg = $GSTMeanDeg + $LonDeg;

// ----------------------------------------------------------------
// Modulate sidereal time angle to fall between 0 and +360 degrees.
   $w = abs($LSTMeanDeg);
   $w = (($LSTMeanDeg < 0)? -1:1)*($w - 360*floor($w/360));
   $w += ($w < 0)? 360:0;
   $LSTMeanDeg = $w;
   $LSTMeanHrs = $LSTMeanDeg / 15;

// -------------------------------------------
// Return mean sidereal time angle in degrees.
   return rtrim(rtrim($LSTMeanDeg,'0'), '.');

}  // End of  Mean_Sid_Time (...)



















// ************************
// ************************
// ANGLE HANDLING FUNCTIONS


/*
   =========================================================================
   This function returns the surface distance between two points on a sphere
   of any given radius, based on their latitude and longitude coordinates.
   This refers to an ideal sphere, but it can also be applied to the Earth
   or moon without any severe error.

   The radius can be given in any convenient units, such as kilometers or
   miles. Whatever units are used for the radius, also apply to the
   returned surface distance.

   ARGUMENTS:
   radius = Radius of the sphere in any convenient units

   Lat1, Lon1 = Latitude and longitude coordinates of 1st point
   Lat2, Lon2 = Latitude and longitude coordinates of 2nd point

*/

   function Dist_Between_Coords ($radius, $Lat1,$Lon1, $Lat2,$Lon2)
{
// Convert degree arguments to radians
   $LatA = deg2rad($Lat1);  $LonA = deg2rad($Lon1);
   $LatB = deg2rad($Lat2);  $LonB = deg2rad($Lon2);

// Compute differences between coordnates.
   $dLat = $LatB - $LatA;
   $dLon = $LonB - $LonA;

// Calculate the angle between the points.
   $w = sin($dLat/2)*sin($dLat/2) + cos($LatA)*cos($LatB)*sin($dLon/2)*sin($dLon/2);
   $a = 2*asin(sqrt($w));

// Compute and return distance in
// the same units as the radius.
   return $radius * $a;
}



// ---------------------------------------------------------------
// This function computes the geodesic distance between two points
// on a sphere from their longitude and latitude coordinates.
// The distance can be returned in kilometers (default) or miles.

   function Geodesic_Dist ($lon1, $lat1, $lon2, $lat2, $KmMi='km')
{
   $KmMiFactor = (strtolower($KmMi) == 'km')? 1:1.609344;

   $ff = 1 / 298.2572221; // Flattening

   $f  = ($lat1 + $lat2) / 2;
   $g  = ($lat1 - $lat2) / 2;
   $l  = ($lon1 - $lon2) / 2;

   $sf = sin($f*M_PI/180);
   $sg = sin($g*M_PI/180);
   $sl = sin($l*M_PI/180);
   $cf = cos($f*M_PI/180);
   $cg = cos($g*M_PI/180);
   $cl = cos($l*M_PI/180);

   $w1 = ($sg * $cl)*($sg * $cl);
   $w2 = ($cf * $sl)*($cf * $sl);
   $s  = $w1 + $w2;

   $w1 = ($cg * $cl)*($cg * $cl);
   $w2 = ($sf * $sl)*($sf * $sl);
   $c  = $w1 + $w2;
   $o  = atan(sqrt($s/$c));
   $r  = sqrt($s*$c) / $o;
   $d  = 2 * $o * 6378.137;

   $h1 = (3*$r-1) / (2*$c);
   $h2 = (3*$r+1) / (2*$s);

   $w1 = ($sf * $cg) * ($sf * $cg) * $h1 * $ff + 1;
   $w2 = ($cf * $sg) * ($cf * $sg) * $h2 * $ff;

   return $d * ($w1 - $w2) * $KmMiFactor;
}




/*
   This function computes the angular diameter of a
   sphere viewed at any distance from its center.

   The equation used accounts for the 3D perspective
   of the curved surface of the sphere vs distance.

   Radius and distance must be measured in same units.

   Returned angle is in degrees.
*/

   function Ang_Diam_at_Dist ($radius, $distance)
  {
   $R = floatval($radius);
   $D = floatval($distance);

// Return FALSE on error, if R*D <= zero.
   if ($R*$D <= 0) {return FALSE;}

// Return angular diameter of sphere, in degrees.
   return rad2deg(2*acos(SqRt($D*$D - $R*$R) / $D));
  }



/*
   ========================================================================
   This function modulates a negative or positive angle to fall within the
   range from 0 to +360 degrees to help handle angles that fall outside
   the standard circular values.  For example, -5 degrees would be
   converted to +355 degrees or 417 degrees would be converted
   to 57 degrees, etc.

   This function uses arbitrary precision arithmetic.


   ARGUMENTS:
   $DegStr = Degrees argument as a numerical string.
   ========================================================================
*/

   function Deg_Mod_360 ($DegStr)
{
   $SignVal = ($DegStr < 0)? -1:1;

   $a = bcmul($SignVal, "$DegStr", 20);

   $b = bcmul($SignVal, bcsub($a, bcmul(360, bcdiv($a, 360)), 20), 20);

   return rtrim(rtrim(bcadd($b, (($b < 0)? 360:0), 16), "0"), ".");

} // End of  Deg_Mod_360()






/*
   Computes the local azimuth (direction) and altitude
   relative to the local horizon, in degrees.

   JDTT    = Astronomical JD argument corresponding to date/time
   RAHrs   = Right Ascension of body, in HOURS - not degrees.
   DeclDeg = Declination of body, from 0 to 90 degrees.

   LongitudeDeg = Geographic longitude (0 to 180 degrees,  Neg/Pos = W/E)
   LatitudeDeg  = Geographic latitude  (0 to 90  degrees,  Neg/Pos = S/N)

   Output  = Horizon coordinates as string = "AzimDeg DeclDeg"
   AzimDeg = Azimuth = Clockwise direction from North = 0  (0 to 360)
   AltDeg  = Altitude relative to horizon (Neg/Pos = Below/Above horizon).
*/

   function Azim_Alt ($JDTT, $RAHrs, $DeclDeg, $LongitudeDeg=0, $LatitudeDeg=0)
{
   $JDTT    = floatval($JDTT);
   $RAHrs   = floatval($RAHrs);
   $DeclDeg = floatval($DeclDeg);
   $LonDeg  = floatval($LongitudeDeg);
   $LatDeg  = floatval($LatitudeDeg);

// Compute local true sidereal time.
   $LSTTrueDeg = Mean_Sid_Time($JDTT, $LonDeg) + dEquinox($JDTT);

// Compute local hour angle of body.
   $H = $LSTTrueDeg - $RAHrs*15;

// Compute azimuth clockwise from 0 to 360 degrees (North = 0 degrees).
   $AzimDeg = 180 + atand2(sind($H), (cosd($H)*sind($LatDeg) - tand($DeclDeg)*cosd($LatDeg)));
   $AzimDeg -= (($AzimDeg >= 360)? 360:0);

// Compute altitude relative to local horizon.  Negative = Below horizon.
   $SinAlt = sind($LatDeg)*sind($DeclDeg) + cosd($LatDeg)*cosd($DeclDeg)*cosd($H);
   $AltDeg = asind($SinAlt);

   return "$AzimDeg $AltDeg";

} // End of  Azim_Alt (...)








/*
   Convert DMS string to decimal degrees.

   Example: Given DMS string = "9 12 50.306"
            Returned = 9.2139738888888889

*/

   function DMS_to_Deg ($DMSstr)
{

// ----------------------------------------------------------
// Read angle/time string argument and force to lower case in
// the event any (deg d h m s :  ' ") symbols are used.

   $dms = strtolower(trim($DMSstr));

// ------------------------------------------------------
// Convert any (deg d m s  ' ") symbols into spaces.

   $dms = str_replace('deg', ' ', $dms);
   $dms = str_replace('d',   ' ', $dms);
   $dms = str_replace('m',   ' ', $dms);
   $dms = str_replace('s',   ' ', $dms);
   $dms = str_replace('',   ' ', $dms);
   $dms = str_replace("'",   ' ', $dms);
   $dms = str_replace('"',   ' ', $dms);
   $dms = str_replace("&deg;",   ' ', $dms);
// ----------------------------------------------------
// Normalize the spacing and then split and extract the
// individual angle/time string elements (dh or hh, mm,ss).

   $dms = preg_replace("/\s+/", " ", trim($dms));
   $wdms = preg_split("[ ]", $dms);
   $wdmsCount = count($wdms);
   $dd = ($wdmsCount >= 1)? bcadd($wdms[0],"0", 20) : "0";
   $mm = ($wdmsCount >= 2)? bcadd($wdms[1],"0", 20) : "0";
   $ss = ($wdmsCount >= 3)? bcadd($wdms[2],"0", 20) : "0";

// ------------------------------------------------
// Remember and then remove any numerical () sign.

   $NumSign = (substr($DMSstr,0,1) == '-')? '-' : '';
   $dd = str_replace('-', '', $dd);
   $dd = str_replace('+', '', $dd);

// -------------------------------------------------------------
// If original angle argument began with a + sign, then preserve
// it so that all returned positive results will have a + sign.
// Otherwise, positive results will NOT have a + sign.

   if (substr($DMSstr,0,1) == '+') {$NumSign = '+';}

// ----------------------------------------------
// Compute decimal degrees/hours value equivalent
// to the given dhms argument elements.

   $w2 = bcadd(bcadd(bcmul($dd,"3600",20),bcmul($mm,"60",20),20),$ss,20);

// -----------------------------------------------------------
// If result equates to zero, then suppress any numerical sign.

   if (bccomp($w2, "0", 20) == 0) {$NumSign = '';}

// ---------------------------------------------------------
// Round off result to 16 decimals, recalling original sign.

   return $NumSign . bcadd(bcdiv($w2,"3600",20), "0.00000000000000005",16);

} // End of  DMS_to_Deg (...)




/*
   ========================================================================
   Convert decimal degrees to degrees, minutes, seconds of arc.
   The angular elements are returned in a space-delimited string.

   Optional symbols are attached to the angle elements. The seconds can
   optionally be rounded to up to 3 decimals with 0=Default.

   Symbols mode = TRUE|FALSE or d|D
   TRUE|d|D means to apply DMS symbols to returned string.
   FALSE means no DMS symbols are applied to returned string.

   ========================================================================
*/

   function Deg_to_DMS ($AngDeg, $ssDecimals=0, $PosSignSymbol='', $SymbolsMode='d')
{

// Init
   $_d_ = $_m_ = $_s_ = '';

// ------------------------------------------------------
// Compute signed angular elements from decimal argument.
// This value is taken as decimal degrees.
   $sign = ($AngDeg < 0)? '-' : '';

   if (($PosSignSymbol === TRUE or $PosSignSymbol == '+') and $sign == '') {$sign = '+';}

   $degrees = abs($AngDeg);
   $dd      = floor($degrees);
   $minutes = 60*($degrees - $dd);
   $mm      = floor($minutes);
   $seconds = 60*($minutes - $mm);
   $ss      = sprintf("%1.3f", $seconds);

// ----------------------------------
// Check for that blasted 60s glitch.
   if ($ss == 60) {$ss = 0; $mm++;}
   if ($mm == 60) {$mm = 0; $dd++;}

// ----------------------------
// Format the angular elements.
   $dd = sprintf("%02d",  $dd);
   $mm = sprintf("%02d",  $mm);
   $ss = sprintf("%1.$ssDecimals" . "f", $ss);

   if ($ss < 10) {$ss = "0$ss";}

// Attach optional DMS symbols.
// Default = TRUE. Can also use 'd' for DMS symbols flag.
   if ($SymbolsMode === TRUE or substr(strtolower($SymbolsMode),0,1) == 'd')
      {
       $_d_ = "&deg;";
       $_m_ = "'";
       $_s_ = '"';
      }

// Done.
   return "$sign$dd$_d_ $mm$_m_ $ss$_s_";

} // End of  Deg_to_DMS (...)





/*
   Converts decimal hours into an hours, minutes
   and seconds string.

   For example, the arguments (10.123456789, 3)
   hours would return the string "10h 07m 24.444s"

   $PosSignSymbol = + if positive value or empty = No sign symbol

   $SymbolsMode = h or :
                  FALSE = No symbols
                  : = Use colons (:) instead of h,m,s symbols
                  h = Use h,m,s symbols instead of colons (= Default).

*/
   function Hours_to_HMS ($hours, $ssDecimals=0, $PosSignSymbol='', $SymbolsMode='h')
{

// ---------------------------
// Initialize symbol carriers.
   $_h_ = $_m_ = $_s_ = '';

   if (trim($PosSignSymbol) == '')  {$PosSignSymbol = FALSE;}

// --------------------------------------------------------------
// Remember original numerical sign and work with absolute value.
   $sign = ($hours < 0)? '-' : '';    $hours = abs($hours);

// -------------------------------------------------------
// Remember numerical sign to be restored on exit, if any.
//   if (($PosSignFlag === TRUE or $PosSignFlag == '+') and $sign == '')
//      {$sign = '+';}

   if (($PosSignSymbol === TRUE or $PosSignSymbol == '+') and $sign == '') {$sign = '+';}

// ---------------------------------------------------
// Compute time elements from absolute hours argument.
   $hh = floor($hours);
   $minutes = 60*($hours - $hh);    $mm = floor($minutes);
   $seconds = 60*($minutes - $mm);  $ss = sprintf("%1.3f",  $seconds);

// -------------------------
// Format the time elements.
   $hh = sprintf("%02d",  $hh);
   $mm = sprintf("%02d",  $mm);
   $ss = sprintf("%1.$ssDecimals" . "f", $ss);

// ----------------------------------
// Patch for that blasted 60s glitch.
   if ($ss == 60) {$ss = 0; $mm++;}
   if ($mm == 60) {$mm = 0; $hh++;}

   $hh = sprintf("%02d",  $hh);
   $mm = sprintf("%02d",  $mm);

   $ss = sprintf("%1.$ssDecimals" . "f", $ss);
   if ($ss < 10) {$ss = "0$ss";}

// -----------------------------------------------
// Attach optional (h m s :) symbols as indicated.
// Default = 'h'
   if ($SymbolsMode == 0 or $SymbolsMode == '')
      {$_h_  =  $_m_  =  $_s_  = ' ';}

   if ($SymbolsMode == 1 or $SymbolsMode == ':')
      {$_h_  =  ':';  $_m_  =  ':';  $_s_  = '';}

   if ($SymbolsMode == 2 or strtolower($SymbolsMode) == 'h')
      {$_h_  =  'h ';  $_m_  =  'm ';  $_s_  =  's';}

// Done.
   return "$sign$hh$_h_$mm$_m_$ss$_s_";

} // End of  Hours_to_HMS(...)


   function HMS_to_Hours ($hhmmss)
{

// -----------------------------------------------------------
// Read hour angle string argument and force to lower case in
// the event any (h m s :) symbols are used.

   $hms = strtolower(trim($hhmmss));

// ------------------------------------------------------
// Convert any (h m s :) symbols into spaces.

   $hms = str_replace('h', ' ', $hms);
   $hms = str_replace('m', ' ', $hms);
   $hms = str_replace('s', ' ', $hms);
   $hms = str_replace(':', ' ', $hms);


// ----------------------------------------------------
// Normalize the spacing and then split and extract the
// individual time string elements (hh, mm,ss).

   $hms = preg_replace("/\s+/", " ", trim($hms));
   $whms = preg_split("[ ]", $hms);
   $whmsCount = count($whms);
   $hh = ($whmsCount >= 1)? bcadd($whms[0],"0", 20) : "0";
   $mm = ($whmsCount >= 2)? bcadd($whms[1],"0", 20) : "0";
   $ss = ($whmsCount >= 3)? bcadd($whms[2],"0", 20) : "0";

// ------------------------------------------------
// Remember and then remove any numerical () sign.

   $NumSign = (substr($hhmmss,0,1) == '-')? '-' : '';
   $hh = str_replace('-', '', $hh);
   $hh = str_replace('+', '', $hh);

// -------------------------------------------------------------
// If original angle argument began with a + sign, then preserve
// it so that all returned positive results will have a + sign.
// Otherwise, positive results will NOT have a + sign.

   if (substr($hhmmss,0,1) == '+') {$NumSign = '+';}

// --------------------------------------
// Compute decimal hours value equivalent
// to the combined hh,mm,ss values.

   $w2 = bcadd(bcadd(bcmul($hh,'3600',20),bcmul($mm,'60',20),20),$ss,20);

// ------------------------------------------------------------
// If result equates to zero, then suppress any numerical sign.

   if (bccomp($w2, '0', 20) == 0) {$NumSign = '';}

// ---------------------------------------------------------
// Round off result to 16 decimals, recalling original sign.

   $w =  bcadd(bcdiv($w2,3600, 20), '0.00000000000000005', 16);

// Trim off any redundant zeros/decimal point.
   return $NumSign . rtrim(rtrim($w, '0'), '.');

} // End of  HMS_to_Hours(...)





/*
   This function rounds off a given time string in decimal
   hours or 'hh:mm:ss.sss' format to the nearest hour,
   minute or second.

   This is primarily for those special cases when rounding
   event times to the nearest hour or minute is sufficient,
   rather than taking it out to a fraction of a second.

   Returns  hh,  hh:mm  or  hh:mm:ss

   FALSE is returned if $hmsMode is invalid.

*/

   function Round_HMS ($hhmmss, $HMSMode='hms')
{
   $HMSMode = strtolower($HMSMode);

// Convert to decimal hours.
   $hours = HMS_to_Hours($hhmmss);

   $nSign = substr($hours, 0,1);
   if ($nSign <> '-' and $nSign <> '+') {$nSign = '';}

   $hms = Hours_to_HMS(abs($hours), 3, '', '');

   list($hh, $mm, $ss) = preg_split("[ ]", $hms);

   $hh = floatval($hh);
   $mm = floatval($mm);
   $ss = floatval($ss);

// Convert minutes+seconds to decimal minutes.
   $minutes = $mm + $ss/60;

// Round to nearest hour.
   if ($HMSMode == 'h')
   {
    return $nSign.round($hh + $minutes/60, 0);
   }

// Round to nearest minute.
   if ($HMSMode == 'hm')
   {
    $mm = round($minutes, 0);
    if($mm == 60) {$hh++; $mm=0;}
    $hh = sprintf("%02d", $hh);
    $mm = sprintf("%02d", $mm);
    return "$nSign$hh:$mm";
   }

// Round to nearest second.
   if ($HMSMode == 'hms')
   {
    $ss = round($ss, 0);
    if($ss == 60) {$mm++; $ss=0;}
    if($mm == 60) {$hh++; $mm=0;}

    $hh = sprintf("%02d", $hh);
    $mm = sprintf("%02d", $mm);
    $ss = sprintf("%02d", $ss);
    return "$nSign$hh:$mm:$ss";
   }
    return FALSE;

} // End of  Round_HMS (...)




function DTZ_to_JD ($DateTimeZoneString)
{
// Set default Time/Zone strings
   $hhmmss = '00:00:00';
   $TZhhmm = '+00:00';

// Parse Date/Time/Zone argument into separate elements.
// Default values are equated to zero.  There MUST BE at
// least one argument (the date).  The Time and Zone are
// optional and default to zero values.
   $DTZStr = strip_wspace($DateTimeZoneString);
   $DTZ = preg_split("[ ]", $DTZStr);

// Count DTZ elements.
   $DTZCount = count($DTZ);

// Parse Date/Time/Zone elements.
   if ($DTZCount >= 1){$yyyymndd = $DTZ[0];}
   if ($DTZCount >= 2){  $hhmmss = $DTZ[1];}
   if ($DTZCount >= 3){  $TZhhmm = $DTZ[2];}

// Parse integer-encoded date element (year, month, day).
   $yyyy = intval(substr($yyyymndd, 0,4));
     $mn = substr($yyyymndd, 4,2); $m = intval($mn);
     $dd = substr($yyyymndd, 6,2); $d = intval($dd);

// Compute JD number corresponding to date elements.
   $JDNum = GregorianToJD($m, $d, $yyyy);

// Parse time elements and compute equivalent fraction of day.
   $TimeFracDay = HMS_to_Hours($hhmmss) / 24;

// Parse time zone elements and compute equivalent fraction of day.
   $TZFracDay = HMS_to_Hours($TZhhmm) / 24;

// Compute final JD (Julian Date) value from the JD number
// and fractional values for the time of day and time zone,
// rounded to 8-decimals.
   $JDate = sprintf("%7.8f", $JDNum - 0.5 + $TimeFracDay - $TZFracDay);

// Done.
   return $JDate;

} // End of  DTZ_to_JD (...)






   function DTZ_to_JDTT ($DateTimeZoneString)
  {
   return Date_Time_Zone_to_JDTT ($DateTimeZoneString);
  }



function Date_Time_Zone_to_JDTT ($DateTimeZoneString)
{
// Set default Time/Zone strings
   $hhmmss = '00:00:00';
   $TZhhmm = '+00:00';

// Parse Date/Time/Zone argument into separate elements.
// Default values are equated to zero.  There MUST BE at
// least one argument (the date).  The Time and Zone are
// optional and default to zero values.
   $DTZStr = strip_wspace($DateTimeZoneString);
   $DTZ = preg_split("[ ]", $DTZStr);

// Count DTZ elements.
   $DTZCount = count($DTZ);

// Parse Date/Time/Zone elements.
   if ($DTZCount >= 1){$yyyymndd = $DTZ[0];}
   if ($DTZCount >= 2){  $hhmmss = $DTZ[1];}
   if ($DTZCount >= 3){  $TZhhmm = $DTZ[2];}

// Parse integer-encoded date element (year, month, day).
   $yyyy = intval(substr($yyyymndd, 0,4));
     $mn = substr($yyyymndd, 4,2); $m = intval($mn);
     $dd = substr($yyyymndd, 6,2); $d = intval($dd);

// Compute JD number corresponding to date elements.
   $JDNum = GregorianToJD($m, $d, $yyyy);

// Parse time elements and compute equivalent fraction of day.
   $TimeFracDay = HMS_to_Hours($hhmmss) / 24;

// Parse time zone elements and compute equivalent fraction of day.
   $TZFracDay = HMS_to_Hours($TZhhmm) / 24;

// Compute Delta T resolved to nearest second.
   $dTFracDay = Delta_T($JDNum)/S_PER_DAY;

// Compute final JD (Julian Date) TT value rounded to 8-decimals.
   $JDTT = sprintf("%7.8f", $JDNum - 0.5 + $TimeFracDay - $TZFracDay + $dTFracDay);

// Done.
   return $JDTT;

} // End of  DTZ_to_JDTT (...)





   function JD_To_Date_Time_String ($JD)
{
// Read general working JD value argument.
   $wJD = floatval($JD);

// Compute corresponding general calendrical JD.
   $CJD = $wJD + 0.5;

// Compute calendrical JD number for date.
   $JDNum = floor($CJD);

// Compute the calendar date elements (mn, dn, Y).
   list($mn, $dn, $Y) = preg_split("[\/]", JDtoGregorian($JDNum));

// Get 3-letter day of week and month
// name abbreviation strings.
   $DoWStr = JDDayOfWeek($JDNum, 2);
   $MonthNameStr = JDMonthName($JDNum, 0);

// Compute time of day as fraction
// of day since beginning of date.
   $TimeFrac = $CJD - $JDNum;

// Compute time elements from the decimal
// value, resolved to nearest second.
   $dd      = 0;
   $hours   = 24*$TimeFrac;
   $hh      = floor($hours);
   $minutes = 60*($hours - $hh);
   $mm      = floor($minutes);
   $seconds = 60*($minutes - $mm);
   $ss      = round($seconds, 0);

// Account for 60s glitch.
   if ($ss == 60){$ss=0;  $mm++;}
   if ($mm == 60){$mm=0;  $hh++;}
   if ($hh == 24){$hh=0;  $dd++;}

// Format time elements into standard
// time string format (hh:mm:ss).
   $hh = sprintf("%02d", $hh);
   $mm = sprintf("%02d", $mm);
   $ss = sprintf("%02d", $ss);
   $hhmmss = "$hh:$mm:$ss";

   return "$Y $MonthNameStr " . sprintf("%02d", $dn) . " $DoWStr $hhmmss";

} // End of  JD_To_Date_Time_String(...)




/*
   ==========================================================
   This is the inverse Julian date function.
   Given a Julian Date, Time Zone Offset and date mode,
   it returns the date and time in either of two modes.

   The date can be returned in 2 modes.
   0 = Integer-encoded date = 19490520 19:21:27
   1 = Alpha date string = "1949 Oct 31 19:21:27 Mon"
*/
   function JD_Z_to_DT ($JDate, $TZhhmm='+00:00', $DateMode=1)
{
// Parse time zone elements and compute
// equivalent fraction of day.
   $TZHM = preg_split("[\:]", trim($TZhhmm));
   $TZHMCount = count($TZHM);
   $hh = $mm = $ss = 0;

   if ($TZHMCount >= 1)
      {
       $hh = $TZHM[0];
       $NegPos = (substr($hh,0,1) == '-')? -1:1;
       $hh = abs(floatval($TZHM[0]));
      }
   if ($TZHMCount >= 2){$mm = floatval($TZHM[1]);}
   if ($TZHMCount >= 3){$ss = floatval($TZHM[2]);}
   $TZFrac = $NegPos * (3600*$hh + 60*$mm) / 86400;

// Compute JD for given time zone.
   $JD = $JDate + $TZFrac;

// Compute JD number.
   $JDNum = floor($JD + 0.5);

// Compute time fraction in hours.
   $hours = 24*($JD - $JDNum + 0.5);

// Compute time elements from time fraction.
   $hh = floor($hours);
   $minutes = 60*($hours - $hh);
   $mm = floor($minutes);
   $seconds = 60*($minutes - $mm);
   $ss = round($seconds, 0);

// Patch for 60s glitch.
   if ($ss == 60) {$mm++;  $ss = 0;}
   if ($mm == 60) {$hh++;  $mm = 0;}

// Format time elements.
   $hh = sprintf("%02d", $hh);
   $mm = sprintf("%02d", $mm);
   $ss = sprintf("%02d", $ss);
   $hhmmss = "$hh:$mm:$ss";

// Compute date elements (m, d, yyyy) corresponding
// to the JD number value.
   list($m, $d, $yyyy) = preg_split("[\/]", JDtoGregorian($JDNum));

// Construct Date/Time string for given time zone.
   $yyyymndd = 10000*$yyyy + 100*$m + $d;

// Construct date string according to DateMode  (1 = Default | 2)
// Mode 1 returns the date in the default integer-encoded Date/Time
// string format.
// EXAMPLE: "19491031 19:21:27"
   if ($DateMode == 1) {return "$yyyymndd $hhmmss";}

// Mode 2 returns the date in alpha numeric Date/Time/DoW string format.
// EXAMPLE: "1949 Oct 31 19:21:27 Mon"
   if ($DateMode == 2)
{
   $Mmm    = substr("JanFebMarAprMayJunJulAugSepOctNovDec", 3*($m-1), 3);
   $dd     = sprintf("%02d", $d);
   $DoWStr = substr("SunMonTueWedThuFriSat", 3*(($JDNum + 1) % 7), 3);

   return "$yyyy $Mmm $dd $DoWStr $hhmmss";
}
   return FALSE; // ERROR = Unknown DateMode

} // End of  JDZ_to_DT (...)




/*
   This function finds the Nth DoW
*/

   function Nth_DoW_of_mY ($n, $DoW, $m, $y)
{
   $JDNum1 = GregorianToJD ($m, 1, $y);
   $DoW1   = ($JDNum1 + 1) % 7;
   $dDoW   = $DoW - $DoW1;
   $JDNumX = $JDNum1 + (($dDoW < 0)? $dDoW + 7 : $dDoW) + 7*($n-1);

   list ($m,$d,$y) = preg_split("[\/]", JDtoGregorian($JDNumX));

   $yyyymndn = sprintf('%04d', $y).sprintf('%02d', $m).sprintf('%02d', $d);

   return $yyyymndn; // Return integer-encoded date.

} // End of  Nth_DoW_of_Ym(...)


/*
   Thisa function finds the last (DoW) of any month of any
   given year, where DoW refers to the day of the week.

   mn = Month number (1 to 12
   DoW values:
   0=Sun, 1=Mon, 2=Tue, 3=Wed, 4=Thu, 5-Fri and 6=Sat

*/

   function Last_DoW_of_mY ($DoW, $mn, $yyyy)
{
// --------------------------
// READ MONTH/YEAR ARGUMENTS.

   $m = intval($mn);
   $y = intval($yyyy);

// ---------------------------------------------
// FIND JD NUMBER OF 1ST DoW OF FOLLOWING MONTH.
// IF DECEMBER, WE FIND THE JD NUMBER OF THE 1ST
// DOW OF JANUARY OF THE FOLLOWING YEAR AND THEN
// SUBTRACT 7.

   $m++;

   if ($m == 12) {$m = 1;   $y++;}

   $JDNum1 = GregorianToJD ($m, 1, $y);
   $DoW1   = ($JDNum1 + 1) % 7;
   $dDoW   = $DoW - $DoW1;

// ---------------------------------------------
// COMPUTE JD NUMBER OF LAST DOW OF GIVEN MONTH.

   $JDNumX = $JDNum1 + (($dDoW < 0)? $dDoW + 7 : $dDoW) - 7;

// ------------------------------------------------
// CONSTRUCT AND RETURN INTEGER-ENCODED DATE VALUE.

   list
  (
   $m,
   $d,
   $y
  ) = preg_split("[\/]", JDtoGregorian($JDNumX));

   return sprintf('%04d', $y).sprintf('%02d', $m).sprintf('%02d', $d);

} // End of  Last_DoW_of_Ym(...)





/*
   THIS FUNCTION RETURN A GREGORIAN DATE STRING CORRESPONDING
   TO ANY GIVEN JULIAN DATE OR JD NUMBER ARGUMENT.

   EXAMPLE:
   2433057 RETURNS THE STRING '1949 May 20 Fri'

   JD NUMBER OR FULL JULIAN DATE (JD) CAN BE USED.
   ONLY THE DATE IS RETURNED.
*/
   function JD_to_Y_Mmm_dn_DoW ($JD)
{
   $JDNum = floor($JD + 0.5);

   list
  (
   $m,
   $d,
   $y
  ) = preg_split("[\/]",JDtoGregorian($JDNum));

   $yyyy = sprintf('%04d', $y);
   $mn   = sprintf('%02d', $m);
   $dn   = sprintf('%02d', $d);

   $MmmStr = substr('JanFebMarAprMayJunJulAugSepOctNovDec', 3*($m-1), 3);
   $DoWStr = substr('SunMonTueWedThuFriSat', 3*(($JDNum + 1) % 7), 3);

   return "$yyyy $MmmStr $dn $DoWStr";

} // End of  JD_Num_to_Y_Mmm_dn(...)




/*
   This function translates an assumed time string into standard form.
   Applies ONLY to purely numerical time strings.


   Time String          Translates to
   1                    01:00:00
   12                   12:00:00
   123                  01:23:00
   1234                 12:34:00
   12345                01:23:45
   123456               12:34:56

Numeric strings are read in 2-digit groups from R to L

Odd length numbers are converted to even length and then
the even length numbers are converted to 6 digit values.

THIS FUMCTION DOES NOT CHECK FOR INVALID TIMES.  IT ONLY
CASTS THE GIVEN NUMERIC TIME STRING IN NORMAL HH:MM:SS
FORMAT AS DEFINED ABOVE.

*/

   function Num_HMS_Trans ($HHMMSS)
{
   $whhmmss = trim($HHMMSS);

// Return default 00:00:00 if empty argument.
   if ($whhmmss == '') {return '00:00:00';}

// If any spaces or colons, attempt to translate as given.
   if (strpos($whhmmss, ':') !== FALSE or strpos($whhmmss, ' ') !== FALSE)
  {
   $hours = HMS_to_Hours($whhmmss);

   while($hours >= 24) {$hours -= 24;}

   return Hours_to_HMS($hours, 0, '', ':');
  }

// Truncate to first six digit characters only.
// Anything beyond the 6th digit will be cut off.
   $whhmmss = substr($whhmmss, 0,6);

// Try to break integer-encoded HHMMSS into 3 groups of 2 digits
// from R to L.  Unspecified elements default to zero strings.
   $n = strlen($whhmmss);

   if ($n == 1 or $n == 3 or $n == 5)
      {$whhmmss = "0$whhmmss";  $n = strlen($whhmmss);}

   for ($i=0;  $i <= $n/2;   $i++) {$whhmmss .= '00';}

// Construct normal standard time string with colons
// between 2-digit groups.
   $hhmmss = '';
   for($i=0;   $i < 3;   $i++)  {$hhmmss .= substr($whhmmss, 2*$i, 2).':';}

// Trim off any redundant ':' character from R end.
// Done.
   return RTrim($hhmmss, ':');

} // End of  Num_HMS_Trans(...)





/*
   CUSTOM FUNCTION SIMILAR TO PHP NATIVE CheckDate() FUNCTION,
   BUT USING A SINGLE INTEGER-ENCODED DATE ARGUMENT VALUE.

   THIS CUSTOMIZED FUNCTION ONLY TESTS FOR DATES INSIDE THE
   600-YEAR SPAN OF THE NASA/DE405 EPHEMERIS MODEL.  ANY
   DATES OUTSIDE THIS RANGE ARE ALSO CONSIDERED INVALID.

   THIS FUNCTION TESTS THE INTEGER-ENCODED DATE FOR VALIDITY
   AND RETURNS A BOOLEAN STATE VALUE (TRUE/FALSE).
   TRUE = OK
   FALSE = INVALID DATE

*/

   function Check_Date($yyyymmdd)
{

// READ INTEGER-ENCODED DATE ARGUMENT
// AND PARSE THE DATE ELEMENT VALUES.

   $w = intval($yyyymmdd);
   $Y = intval(substr($w,0,4));
   $m = intval(substr($w,4,2));
   $d = intval(substr($w,6,2));

// CHECK DATE FOR VALIDITY AND RETURNS TRUE/FALSE.
// TRUE  = OK
// FALSE = INVALID DATE

   return CheckDate($m, $d, $Y);

} // END OF  Check_Date(yyyymndn)






/*
   THIS FUNCTION TESTS A GIVEN hhmmss TIME STRING FOR VALIDITY
   AND RETURNS A BOOLEAN STATE VALUE (TRUE/FALSE).
   TRUE  = OK
   FALSE = INVALID TIME

   FOR A TIME TO BE VALID, THE SUM OF THE
   ELEMENTS MUST BE >= 0 AND <= 24 HOURS.

*/

   function Check_Time($hhmmss)
{

// READ STANDARD hh:mm:ss TIME STRING ARGUMENT.
// TIME ELEMENTS MUST NOT SUM TO > 24 HOURS AND
// MUST NOT BE A NEGATIVE VALUE.

   $w = HMS_to_Hours($hhmmss);

// CHECK TIME FOR VALIDITY AND RETURN TRUE/FALSE.
// TRUE  = OK
// FALSE = INVALID TIME (< 0 or > 24 HOURS)

   return ($w < 0 or $w > 24)? FALSE:TRUE;

} // END OF  Check_Time(hhmmss)





/*
   ========================================================
   THIS FUNCTION TESTS A GIVEN LATITUDE STRING FOR VALIDITY
   AND RETURNS A BOOLEAN STATE VALUE (TRUE/FALSE).
   TRUE  = OK
   FALSE = INVALID LATUTUDE VALUE

   VALID RANGE IS FROM -90 TO +90 DEGREES.  ANYTHING OUTSIDE
   THIS VALID RANGE RETURNS FALSE.

   LATITUDE ARGUMENT CAN BE DECIMAL DEGREES OR A DMS STRING.
*/
   function Check_Lat($LatStr)
  {
   $w = DMS_to_Deg($LatStr);

   return ($w < -90 or $w > 90)? FALSE:TRUE;
  }



/*
   =========================================================
   THIS FUNCTION TESTS A GIVEN LONGITUDE STRING FOR VALIDITY
   AND RETURNS A BOOLEAN STATE VALUE (TRUE/FALSE).
   TRUE  = OK
   FALSE = INVALID LONGITUDE

   VALID RANGE IS FROM -360 TO +360 DEGREES.  ANYTHING OUTSIDE
   THIS VALID RANGE RETURNS FALSE.

   LONGITUDE ARGUMENT CAN BE DECIMAL DEGREES OR A DMS STRING.
*/
   function Check_Lon($LonStr)
  {
   $w = DMS_to_Deg($LonStr); return ($w < -360 or $w > 360)? FALSE:TRUE;
  }















   function Interp_Lon ($PhaseInterpTable, $JDTTArg)
{
   $wArray = preg_split("[\n]", trim($PhaseInterpTable));
   $wCount = count($wArray);

   $JDArg = floatval($JDTTArg);

   for ($i=1;   $i < $wCount;   $i++)
   {
   $CurrLine = strip_wspace($wArray[$i]);
   $PrevLine = strip_wspace($wArray[$i-1]);

   list($CurrJDTT, $CurrPhaseDeg) = preg_split("[ ]", $CurrLine);
   list($PrevJDTT, $PrevPhaseDeg) = preg_split("[ ]", $PrevLine);

   $diff = $CurrPhaseDeg - $PrevPhaseDeg;

   if ($diff < 0)
      {
       $CurrPhaseDeg += 360;
       $wArray[$i] = "$CurrJDTT $CurrPhaseDeg\n";
      }
   }

// Construct final interpolation table ready for use.
   $w = '';
   for ($i=0;   $i < $wCount;   $i++)
   {
   $w .= trim($wArray[$i]) . "\n";
   }
   $PhaseInterpTableStr = trim($w);

// Perform interpolation for phase angle corresponding to JD argument.
   $PhaseDeg = Lagrange_interp($PhaseInterpTableStr, $JDArg);

   $PhaseDeg -= ($PhaseDeg > 360)? 360:0;

   return sprintf("%1.5f", $PhaseDeg);

} // End of  Interp_Lon(...)



// -----------------------------------------------------------
// IDENTICAL TO Interp_Lon() TO REPLACE IT IN FUTURE PROGRAMS.

   function Interp_Angle ($PhaseInterpTable, $JDTTArg)
{
   $wArray = preg_split("[\n]", trim($PhaseInterpTable));
   $wCount = count($wArray);

   $JDArg = floatval($JDTTArg);

   for ($i=1;   $i < $wCount;   $i++)
   {
   $CurrLine = strip_wspace($wArray[$i]);
   $PrevLine = strip_wspace($wArray[$i-1]);

   list($CurrJDTT, $CurrPhaseDeg) = preg_split("[ ]", $CurrLine);
   list($PrevJDTT, $PrevPhaseDeg) = preg_split("[ ]", $PrevLine);

   $diff = $CurrPhaseDeg - $PrevPhaseDeg;

   if ($diff < 0)
      {
       $CurrPhaseDeg += 360;
       $wArray[$i] = "$CurrJDTT $CurrPhaseDeg\n";
      }
   }

// CONSTRUCT INTERPOLATION TABLE.
   $w = '';
   for ($i=0;   $i < $wCount;   $i++)
   {
   $w .= trim($wArray[$i]) . "\n";
   }
   $PhaseInterpTableStr = trim($w);

// -------------------------------------------------------------------
// PERFORM INTERPOLATION FOR PHASE ANGLE CORRESPONDING TO JD ARGUMENT.

   $PhaseDeg = Lagrange_Interp($PhaseInterpTableStr, $JDArg);

   $PhaseDeg -= ($PhaseDeg > 360)? 360:0;

   return sprintf("%1.5f", $PhaseDeg);

} // End of  Interp_Angle(...)






/*
  THIS FUNCTION RETURNS AN HTML TEXTAREA READY FOR DISPLAY
  ACCORDING TO THE GIVEN PARAMETERS.

*/
   function Text_Area ($TextBlock, $TxtRows='', $TxtCols=81, $ColorStr='black', $BGColorStr='LightYellow')
 {
   $rows     = intval($TxtRows);
   $cols     = intval($TxtCols);
   $C        = trim($ColorStr);
   $B        = trim($BGColorStr);

   if ($cols <= 0)        {$cols = 1;}
   if ($ColorStr   == '') {$ColorStr = 'black';}
   if ($BGColorStr == '') {$BGColorStr = 'white';}

   if (intval($rows) == 0) {$rows = 2 + substr_count($TextBlock, "\n");}

// -----------------------------
// CONSTRUCT HTML TEXTAREA BLOCK

   return
"<textarea rows='$rows' cols='$cols' readonly='readonly'
style='color:$ColorStr; background-color:$BGColorStr; font-family:monospace;
font-size:16px; font-weight:bold; border:1px solid black; padding:6px;'>
$TextBlock
</textarea>";

 } // end of  Text_Area(...)



/*
  THIS FUNCTION RETURNS AN HTML TEXTAREA TO DISPLAY PHP
  PROGRAMMING CODE ACCORDING TO THE GIVEN PARAMETERS.

*/
   function Text_Area_PHP ($TextBlock, $TxtRows='', $TxtCols=81, $ColorStr='black', $BGColorStr='white')
 {
   $rows     = intval($TxtRows);
   $cols     = intval($TxtCols);
   $C        = trim($ColorStr);
   $B        = trim($BGColorStr);

   if ($cols <= 0)        {$cols = 1;}
   if ($ColorStr   == '') {$ColorStr = 'black';}
   if ($BGColorStr == '') {$BGColorStr = 'white';}

   if (intval($rows) == 0) {$rows = 2 + substr_count($TextBlock, "\n");}

// -----------------------------
// CONSTRUCT HTML TEXTAREA BLOCK

   return
"<textarea rows='$rows' cols='72' readonly='readonly'
style='color:$ColorStr; background-color:$BGColorStr; font-family:monospace;
font-size:16px; font-weight:bold; border:2px solid red; padding:6px;'>
$TextBlock
</textarea>";

 } // end of  Text_Area_PHP(...)







/*

COMPUTE USA DAYLIGHT SAVING TIME START/END DATES.

VALID YEAR RANGE: 1975 to 9999

RETURNS A STRING LIKE:
2017 Mar 12 Sun to 2017 Nov 05 Sun = 238 days = 34 weeks = 2457825 to 2458063

DEPENDING ON THE YEAR, THREE DIFFERENT RULES ARE USED.

*/

   function USA_DST ($year)
{
   $y = intval($year);

   if ($y < 1975 or $y > 9999)
      {return "ERROR: Invalid Year '$y'.  Valid range: 1975 to 9999";}
   if ($y > 1974 and $y < 1987)
  {
   $start = Last_DoW_of_mY (0,  4, $y); // Last Sunday in April
   $end   = Last_DoW_of_mY (0, 10, $y); // Last Sunday in October
  }
   if ($y > 1986 and $y < 2007)
  {
   $start = Nth_DoW_of_mY  (1,  0, 4, $y); // First Sunday in April
   $end   = Last_DoW_of_mY (0, 10, $y);    // Last Sunday in October
  }
   if ($y > 2006)
  {
   $start = Nth_DoW_of_mY (2,  0,  3, $y); // Second Sunday in March
   $end   = Nth_DoW_of_mY (1,  0, 11, $y); // First Sunday in November
  }

   $JDNumStart = 0 + DTZ_to_JD("$start 12");
   $StartDate  = JD_to_Y_Mmm_dn_DoW($JDNumStart);
   $JDNumEnd   = 0 + DTZ_to_JD("$end 12");
   $EndDate    = JD_to_Y_Mmm_dn_DoW($JDNumEnd);
   $DaysSpan   = $JDNumEnd - $JDNumStart;
   $WeeksSpan  = $DaysSpan / 7;

   return "$StartDate to $EndDate = $DaysSpan days = $WeeksSpan weeks = $JDNumStart to  $JDNumEnd ";

} // end of  USA_DST(...)





/*

COMPUTE EUROPEAN UNION SUMMER TIME START/END DATES.

RETURNS A STRING LIKE:
2017 Mar 26 Sun to 2017 Oct 29 Sun = 217 days = 31 weeks = 2457839 to 2458056

*/
   function EU_ST ($year)
{
   $y = intval($year);

   if ($y < 2002 or $y > 9999)
      {return "ERROR: Invalid Year '$y'.  Valid range: 2002 to 9999";}

   $start = Last_DoW_of_mY (0,  3, $y); // Last Sunday in March
   $end   = Last_DoW_of_mY (0, 10, $y); // Last Sunday in October

   $JDNumStart = 0 + DTZ_to_JD("$start 12");
   $StartDate  = JD_to_Y_Mmm_dn_DoW($JDNumStart);
   $JDNumEnd   = 0 + DTZ_to_JD("$end 12");
   $EndDate    = JD_to_Y_Mmm_dn_DoW($JDNumEnd);
   $DaysSpan   = $JDNumEnd - $JDNumStart;
   $WeeksSpan  = $DaysSpan / 7;

   return "$StartDate to $EndDate = $DaysSpan days = $WeeksSpan weeks = $JDNumStart to  $JDNumEnd ";

}



/*
   ============================================================================
   GEOCENTRIC EQUARTORIAL LUNAR EPHEMERIS STATISTICS FUNCTION

   THIS FUNCTION RETURNS THE BASIC GEOCENTRIC EQUATORIAL LUNAR EPHEMERIS
   STATISTICS FOR ANY SINGLE GIVEN JD ARGUMENT ON THE TT SCALE.

   THE COMPUTATIONS ARE BASED ON NASA/JPL DEVELOPMENT EPHEMERIS (DE405) MODEL
   SPANNING THE 600-YEAR PERIOD FROM 1600 TO 2200.


   JDTTArg = Julian Date argument on TT scale for computation corresponding to
             the date, time and time zone that were used to compute it.

   TableMode 1 = Output angles formatted as raw decimal values (default).
   TableMode 2 = Output angles formatted as HMS/DMS strings.

   ============================================================================
*/

   function Geocentric_Equ_Moon_Stats ($JDTTArg, $TableMode=1)
{
// -------------------
// READ JDTT ARGUMENT.

   $JDTT = floatval($JDTTArg);

// -----------------------------------
// READ DATA TABLE INTO WORKING ARRAY.

   $wArray = file('data/Moon_DE405_Geo_Equ_Coords_1600_to_2200.data');
   $wCount = count($wArray);

// ----------------------------------------------------
// FIND INDEX OF FIRST JDTTx10 VALUE IN DATA TABLE THAT
// IS GREATER THAN TargetJDx10 AND GET SEVEN DATA LINES
// CENTERED ON THE INDEXED LINE.

   $TargetJDx10 = round(10*$JDTTArg, 0);

// -------------------------------------------------
// FLAG TO INDICATE IF JDTTArg IS WITHIN DATA RANGE.
// THIS FLAG WILL BE SET TO TRUE ON EXIT IF JDTTArg
// IS VALID.

   $FoundFlag = FALSE; // INITIALIZE FLAG BEFORE SEARCH.

   for ($i=0;   $i < $wCount;   $i++)
  {
   $CurrJDx10 = substr(trim($wArray[$i]), 0,8);

   if ($CurrJDx10 > $TargetJDx10) {$FoundFlag = TRUE;  break;}
  }

// -------------------------------------------
// DEFINE AND INITIALIZE INTERPOLATION TABLES.

   $RAInterpTableStr   = $DeclInterpTableStr  = '';
   $DistInterpTableStr = $PhaseInterpTableStr = '';

// ----------------------------------------------
// CREATE INTERPOLATION TABLES OF TIME vs VALUES.

   for ($j = $i-4;   $j < $i+3;   $j++)
  {
   $CurrLine = trim($wArray[$j]);
   $wJDTT     = intval(substr($CurrLine,  0,8)) / 10;
   $wRAHrs    = intval(substr($CurrLine,  8,9)) / 10000000;
   $wDeclDeg  = intval(substr($CurrLine, 17,9)) / 1000000;
   $wDistKm   = intval(substr($CurrLine, 26,9)) / 1000;
   $wPhaseDeg = intval(substr($CurrLine, 35,8)) / 100000;

// ------------------------------------------------
// APPEND LINES TO RESPECTIVE INTERPOLATION TABLES.

   $RAInterpTableStr    .= "$wJDTT $wRAHrs\n";
   $DeclInterpTableStr  .= "$wJDTT $wDeclDeg\n";
   $DistInterpTableStr  .= "$wJDTT $wDistKm\n";
   $PhaseInterpTableStr .= "$wJDTT $wPhaseDeg\n";
  }

// ----------------------------------------------
// PERFORM INTERPOLATIONS FOR JDTT TIME ARGUMENT.

   $RAHrs    = Interp_Hours($RAInterpTableStr, $JDTT);
   $DeclDeg  = Lagrange_Interp($DeclInterpTableStr, $JDTT);
   $DistKm   = Lagrange_Interp($DistInterpTableStr, $JDTT);
   $PhaseDeg = Interp_Lon($PhaseInterpTableStr, $JDTT);

// -------------------------------------------------------
// FORMAT VALUES FOR TABULATION.  FIELDS LEFT-PADDED WITH
// SPACES TO 12 CHARACTERS EXCEPT FOR THE JDTT VALUE.

   $JDTT     = sprintf("%7.8f", $JDTT);
   $RAHrs    = '  '.sprintf("%2.7f",  $RAHrs).'   ';
   $DeclDeg  = ' '.sprintf("%+2.6f", $DeclDeg).' ';
   $RAHMS    = L_Pad_Text(Hours_to_HMS($RAHrs, 3,  '', ''), 14);
   $DeclDMS  = L_Pad_Text(Deg_to_DMS($DeclDeg, 2, '+', ''), 12);
   $DistKm   = L_Pad_Text(sprintf("%6.3f",  $DistKm),   11);
   $PhaseDeg = ' '.sprintf("%2.5f",  $PhaseDeg).' ';

// -------------------------------------------------------
// CONSTRUCT RETURNED TABLE LINE ACCORDING TO (TableMode).
// RETURNS DECIMAL OR SEXAGESIMAL FORMATED VALUES.
// LOGIC: IF NOT TABLE MODE 2 THEN TABLE MODE 1.
// TableMode = 1 = Raw decimal values = Default
// TableMode = 2 = Sexagesimal format = HMS/DMS

   if ($TableMode == 2)
      {$RA = $RAHMS;  $Decl = $DeclDMS;}
   else
      {$RA = $RAHrs;  $Decl = $DeclDeg;}

// ----------------------------------------
// DONE. RETURN FORMATTED STATISTICS LINE.

   return "$JDTT   $RA    $Decl $DistKm $PhaseDeg";

} // end of  Geocentric_Equ_Moon_Stats(...)






/*
   THIS FUNCTION RETURNS A TEXT DESCRIPTION OF
   THE LUNAR PHASE FOR A GIVEN PHASE ANGLE.
   THESE VALUES MAY BE ADJUSTED IF THEY SEEM A
   BIT OFF.  THEY WERE ARBITRARILY ASSIGNED.

   A PREVIOUS VERSION USED if(...) STRUCTURES.

   THE NORMAL BREAKS ARE OMITTED BECAUSE THEY
   ARE NOT NEEDED FOLLOWING THE TERMINAL RETURN.
   THEY WOULD NEVER BE REACHED ANYWAY.
*/

  function Lunar_Phase_to_Text ($PhaseDegrees)
{
 $a = round($PhaseDegrees, 0);

 switch ($a)
{
 case ($a >=  0 and $a <=   9):return "New moon.";
 case ($a >   9 and $a <=  29):return "Waxing crescent, just past new moon.";
 case ($a >  29 and $a <=  76):return "Waxing crescent moon.";
 case ($a >  76 and $a <=  88):return "Waxing crescent moon, approaching first quarter.";
 case ($a >  88 and $a <= 101):return "First quarter moon.";
 case ($a > 101 and $a <= 110):return "Waxing gibbous moon, just past first quarter.";
 case ($a > 110 and $a <= 159):return "Waxing gibbous moon.";
 case ($a > 159 and $a <= 170):return "Waxing gibbous, approaching full moon.";
 case ($a > 170 and $a <= 192):return "Full moon.";
 case ($a > 192 and $a <= 198):return "Waning gibbous, just past full moon.";
 case ($a > 198 and $a <= 250):return "Waning gibbous moon.";
 case ($a > 250 and $a <= 260):return "Waning gibbous moon, approaching last quarter.";
 case ($a > 260 and $a <= 271):return "Last quarter moon.";
 case ($a > 271 and $a <= 274):return "Waning crescent moon, just past last quarter.";
 case ($a > 274 and $a <= 330):return "Waning crescent moon.";
 case ($a > 330 and $a <= 349):return "Waning crescent, approaching new moon.";
 case ($a > 349 and $a <= 360):return "New moon.";

 default: return FALSE; // INDICATES ERROR: ANGLE OUTSIDE 0 TO +360 deg RANGE.

} // END OF  switch(a)

} // END OF  Lunar_Phase_to_Text(...)








/*
   ============================================================================
   GEOCENTRIC ECLIPTICAL LUNAR EPHEMERIS STATISTICS FUNCTION

   THIS FUNCTION RETURNS THE BASIC GEOCENTRIC ECLIPTICAL LUNAR EPHEMERIS
   STATISTICS FOR ANY SINGLE GIVEN JD ARGUMENT ON THE TT SCALE.

   THE COMPUTATIONS ARE BASED ON NASA/JPL DEVELOPMENT EPHEMERIS (DE405) MODEL
   SPANNING THE 600-YEAR PERIOD FROM 1600 TO 2200.


   JDTTArg = Julian Date argument on TT scale for computation corresponding to
             the date, time and time zone that were used to compute it.

   DE405_Lunisolar_Geo_Ecl_Coords_1600_2200.data


GIVEN

i          = Array index of any given data line
DataOrigin = Array index of first numerical data line
JDOrigin   = Julian Date corresponding to first data line
One AU     = 149597870.700 km (IAU/NASA/JPL DE430)

The tabular JD corresponding to any general index (i) value
within the data array may be found by:

TJD   = JDOrigin + (i - DataOrigin)/2
and
i = DataOrigin + 2*(TJD - JDOrigin)


FOR THIS TABLE

TJD = 2305447.5 + (i - DataOrigin)/2

and

i = DataOrigin + 2*(TJD - 2305447.5)


DATA VALUES

Bs, Bm = Geocentric ecliptical latitude of sun and moon
Ls, Lm = Geocentric ecliptical longitude of sun and moon
DistAU = Geocentric sun distance in Astronomical Units
DistKm = Geocentric moon distance in kilometers

Lm(0,  8, D5) Bm(8,  7, D5) DistKm(15,  9, D3)
Ls(24, 8, D5) Bs(32, 7, D5) DistAU(39, 10, D9)

10477140+14171938311842227998414+0000020983154202
11136666+08258138519983928049371+0000000983164864
11788540+02299538734797528100328-0000020983177149
12432743-03622938952616828151283-0000040983191044
13069396-09434139169503628202237-0000060983206532
13698756-15066239381357728253191-0000070983223592
14321212-20458539584029728304143-0000090983242202

   ============================================================================
*/

   function Geocentric_Ecl_Lunisolar_Stats ($JDTTArg)
{
// -------------------
// READ JDTT ARGUMENT.

   $JDTT = floatval($JDTTArg);

   $J = floor($JDTT);
   $f = $JDTT - $J;

   if ($f <= 0.25) {$f = 0;}
   if ($f > 0.25 and $f < 0.75) {$f = 0.5;}
   if ($f >= 0.75) {$f = 1.0;}

   $TJDTT = sprintf("%7.1f", $J + $f);

// -------------------------------
// LOAD DATA FILE INTO WORK ARRAY.

   $DataArray  = file('data/DE405_Lunisolar_Geo_Ecl_Coords_1600_to_2200.data');
   $DataCount  = count($DataArray);
   $JDTTOrigin = 2305447.5;

// ----------------------------------------------------------
// FIND INDEX OF DATA ORIGIN (1st LINE STARTING WITH A DIGIT)

   $FoundFlag = FALSE;
   for($j=0;   $j < $DataCount;   $j++)
  {
   if (is_numeric(substr($DataArray[$j],0,1)))
      {$DataOrigin = $j;  $FoundFlag = TRUE;  break;}
  }

// ----------------------------------------------------------
// COMPUTE ARRAY INDEX OF CLOSEST DATA LINE TO JDTT ARGUMENT.

   $index = $DataOrigin + 2*($TJDTT - $JDTTOrigin);

// -----------------------------------------------
// GET 7 DATA LINES CENTERED ON INDEXED DATA LINE.
$LunisolarTable = '';

for($j=$index-3;   $j < $index+4;   $j++)
{
$CurrLine = "$DataArray[$j]";

$TJD = sprintf("%7.1f", ($JDTTOrigin + ($j - $DataOrigin)/2));

/*
Lm(0,  8, D5) Bm(8,  7, D5) DistKm(15,  9, D3)
Ls(24, 8, D5) Bs(32, 7, D5) DistAU(39, 10, D9)

10477140+14171938311842227998414+0000020983154202
*/

// Extract raw data line elements.
   $dp = 5;
   $w = substr($CurrLine, 0, 8);
   $Lm = substr($w, 0, strlen($w)-$dp) .'.'. substr($w, strlen($w)-$dp, strlen($w));

   $dp = 5;
   $w = substr($CurrLine, 8, 7);
   $Bm = substr($w, 0, strlen($w)-$dp) .'.'. substr($w, strlen($w)-$dp, strlen($w));

   $dp = 3;
   $w = substr($CurrLine, 15, 9);
   $DistKm = substr($w, 0, strlen($w)-$dp) .'.'. substr($w, strlen($w)-$dp, strlen($w));

   $dp = 5;
   $w = substr($CurrLine, 24, 8);
   $Ls = substr($w, 0, strlen($w)-$dp) .'.'. substr($w, strlen($w)-$dp, strlen($w));

   $dp = 5;
   $w = substr($CurrLine, 32, 7);
   $Bs = substr($w, 0, strlen($w)-$dp) .'.'. substr($w, strlen($w)-$dp, strlen($w));

   $dp = 9;
   $w = substr($CurrLine, 39, 10);
   $DistAU = substr($w, 0, strlen($w)-$dp) .'.'. substr($w, strlen($w)-$dp, strlen($w));

// COMPUTE LUNAR PHASE ANGLE.
   $w = 360 - $Lm + $Ls;
   $PhaseDeg = substr("000000000" . sprintf("%1.5f", 360 - ($w -= ($w > 360)? 360:0)), -9);

// APPEND CURRENT LINE TO WORKING TABLE.
   $LunisolarTable .= "$TJD $Lm $Bm $DistKm $PhaseDeg $Ls $Bs $DistAU\n";
}
   $LunisolarTable = trim($LunisolarTable);



// AT THIS POINT THE ($LunisolarTable) IS READY FOR USE
// IN CONSTRUCTING THE INTERPOLATION TABLES FOR COMPUTING
// THE STATISTICS FOR THE GIVEN JDTT ARGUMENT.

// -------------------------------------------
// DEFINE AND INITIALIZE INTERPOLATION TABLES.

   $LmInterpTableStr     = $BmInterpTableStr       = '';
   $DistKmInterpTableStr = $PhaseDegInterpTableStr = '';
   $LsInterpTableStr     = $BsInterpTableStr       = '';
   $DistAUInterpTableStr = '';

   $wArray = preg_split("[\n]", $LunisolarTable);
   $wCount = count($wArray);

   for ($i=0;   $i < $wCount;   $i++)
{
   $CurrLine = trim($wArray[$i]);
   list
  (
   $TJDTT,
   $Lm, $Bm, $DistKm, $PhaseDeg,
   $Ls, $Bs, $DistAU
  )= preg_split("[ ]", strip_wspace($CurrLine));

// APPEND CURRENT INTERPOLATION TABLE LINES
   $LmInterpTableStr       .= "$TJDTT $Lm\n";
   $BmInterpTableStr       .= "$TJDTT $Bm\n";
   $DistKmInterpTableStr   .= "$TJDTT $DistKm\n";
   $PhaseDegInterpTableStr .= "$TJDTT $PhaseDeg\n";
   $LsInterpTableStr       .= "$TJDTT $Ls\n";
   $BsInterpTableStr       .= "$TJDTT $Bs\n";
   $DistAUInterpTableStr   .= "$TJDTT $DistAU\n";
}

// PERFORM INTERPOLATIONS.

// ----------------------------------------------
// PERFORM INTERPOLATIONS FOR JDTT TIME ARGUMENT.

// COMPUTE GEOCENTRIC ECLIPTICAL COORDINATES AND PHASE OF MOON
   $Lm       = Interp_Lon($LmInterpTableStr, $JDTT);
   $Bm       = Lagrange_Interp($BmInterpTableStr, $JDTT);
   $DistKm   = Lagrange_Interp($DistKmInterpTableStr, $JDTT);
   $PhaseDeg = Interp_Lon($PhaseDegInterpTableStr, $JDTT);

// COMPUTE GEOCENTRIC ECLIPTICAL COORDINATES OF SUN
   $Ls       = Interp_Lon($LsInterpTableStr, $JDTT);
   $Bs       = Lagrange_Interp($BsInterpTableStr, $JDTT);
   $DistAU   = Lagrange_Interp($DistAUInterpTableStr, $JDTT);

// ---------------------------------------------------------
// FORMAT VALUES FOR TABULATION.
// OUTPUT WILL BE SINGLE-SPACE-DELIMITED NUMERICAL ELEMENTS
// IN A STRING.

   $JDTT     = sprintf("%7.8f",  $JDTT);
   $Lm       = sprintf("%1.5f",  $Lm);
   $Bm       = sprintf("%+1.5f", $Bm);
   $DistKm   = sprintf("%6.3f",  $DistKm);
   $PhaseDeg = sprintf("%1.5f",  $PhaseDeg);
   $Ls       = sprintf("%1.5f",  $Ls);
   $Bs       = sprintf("%+1.5f", $Bs);
   $DistAU   = sprintf("%1.9f",  $DistAU);

   $out = "$JDTT $Lm $Bm $DistKm $PhaseDeg $Ls $Bs $DistAU";

// -------------------------------------------
// DONE.  RETURN ECLIPTICAL STATISTICS STRING.

   unset($DataArray);
   unset($wArray);

   return trim($out);

} // END OF  Geocentric_Ecl_Lunisolar_Stats(JDTTArg)


/* ----------------------------------------------------
   Define a name for the cookie and set expiration date
   to 30 days if not used or deleted before then.  This
   value is expressed in seconds from the time that the
   cookie is created (Time Now + days*86400 seconds).
*/
   $CookieName = 'Current_Moon_Sun_OverLay';

   $SetToExpireIn30Days = time() + 30*86400;

   $RandomFlag = FALSE;



   if($RandomFlag === TRUE)
   {
     $PageHeaderText = 'Refresh This Page For A New Random Date/Time/Zone example.';
     $yyyymndn = Rand_Int_Enc_Date();
     $hhmmss   = Rand_Time_String();
     $TZhhmm   = Hours_to_HMS(HMS_to_Hours(mt_rand(0, 12)), 0, '+', ':');
     if (mt_rand() % 2 == 0) {$TZhhmm = str_replace("+", "-", $TZhhmm);}
   }
else
  {
   $PageHeaderText = "<div style='font-family:Verdana; text-align:center;'><br><b>CURRENT MOON/SUN OVERLAY SIZE COMPARISON</b><br><br>REFRESH This page to update to the current moment.<br>It will take a long time for any change to become visually apparent.<br><br><span style='font-size:80%;'>PHP Program by Jay Tanner</span>
</div>";
   $yyyymndn = gmdate('Ymd');
   $hhmmss   = gmdate('H:i:s');
   $TZhhmm   = '+00:00:00';
  }

// CHECK FOR TESTING MODE.
// FOR GENERAL TESTING, INSERT ANY OVERRIDE DATE/TIME/TZ HERE INSTEAD.
   if ($InTestingMode === TRUE)
  {
   $yyyymndn = 19490520;
   $hhmmss   = '13:52:27';
   $TZhhmm   = '+00:00';
  }

// PARSE GIVEN DATE ELEMENTS.
   $yyyy = substr($yyyymndn, 0, 4);  $Y = intval($yyyy);
   $mn   = substr($yyyymndn, 4, 2);  $m = intval($mn);
   $dn   = substr($yyyymndn, 6, 2);  $d = intval($dn);



// -------------------------------------------------
// Get strings for month and DoW name abbreviations.
   $JDNumLT = GregorianToJD($m, $d, $Y);
   $MmmStr  = substr('JanFebMarAprMayJunJulAugSepOctNovDec', 3*($m-1), 3);
   $DoWStr  = substr('SunMonTueWedThuFriSat', 3*(($JDNumLT + 1) % 7), 3);

   $JDTT = DTZ_to_JDTT("$yyyymndn $hhmmss $TZhhmm");

// COMPUTE NASA DELTA T FOR JDTT.
   $dTSec = Delta_T($JDTT);
   $dThhmmss = Hours_to_HMS($dTSec/3600, 0, '+', ':');


// COMPUTE GEOCENTRIC LUNAR ECLIPTICAL EPHEMERIS STATISTICS FOR JDTT.

   $EclipticalStatsStr = Geocentric_Ecl_Lunisolar_Stats ($JDTT);

// PARSE ECLIPTICAL LUNISOLAR STATISTICS
   list
  (
   $TJDTT,
   $Lm, $Bm, $Distkm, $PhaseDeg,
   $Ls, $Bs, $DistAU
  ) = preg_split("[ ]", trim($EclipticalStatsStr));

// COMPUTE GEOCENTRIC LUNAR AND SOLAR ANGULAR DIAMETERS IN DEGREES.
// BOTH SHOUD BE APPROXIMATELY 0.5 DEGREE IN ANGULAR DIAMETER.
   $AngDiamMoon = 360*atan(1738.1 / $Distkm) / M_PI;
   $AngDiamSun  = 2*rad2deg(asin(696000 / ($DistAU * KM_PER_AU)));

// COMPUTE ABSOLUTE DIFFERENCE IN SIZES IN DEG, MIN AND SEC OF ARC.
   $AngDiamDiffDeg = abs($AngDiamMoon - $AngDiamSun);
   $AngDiamDiffDMS = Deg_to_DMS($AngDiamDiffDeg, 1, '', 'd');
   $AngDiamDiffDMS = substr($AngDiamDiffDMS, -9);
   $AngDiamDiffDeg = sprintf("%1.6f", $AngDiamDiffDeg);

// FORMAT VALUES FOR TABULATION.
   $AngDiamMoon = sprintf("%1.8f", $AngDiamMoon);
   $AngDiamSun  = sprintf("%1.8f", $AngDiamSun);
   $Distmi      = sprintf("%6.3f", $Distkm/1.609344);
   $Distkm      = sprintf("%6.3f", $Distkm);
   $PhaseDeg    = sprintf("%1.3f", $PhaseDeg);

   $DistAU      = sprintf("%1.9f", $DistAU);
   $DistKm      = number_format($DistAU * KM_PER_AU);
   $DistMi      = number_format($DistAU * MI_PER_AU);

// CONSTRUCT FULL DATE TEXT STRING.
   $DateString  = "$Y $MmmStr $dn $DoWStr";

// STORE ANGULAR DIAMETERS OF MOON AND SUN IN A COOKIE FOR
// USE IN SUBSEQUENT COMPARATIVE DISK SIZE COMPUTATIONS.
   $CookieDataString = "$AngDiamMoon|$AngDiamSun|$DateString|$hhmmss|$TZhhmm|$dThhmmss|$JDTT|$Distkm|$Distmi|$DistAU";
   setcookie ($CookieName, $CookieDataString, $SetToExpireIn30Days);

// REFORMAT VALUES FOR TABULATION.
   $Distkm = number_format($Distkm, 0);
   $Distmi = number_format($Distmi, 0);

/*
   Display the computations and generate a PNG image of
   the moon and sun discs overlapping so their geocentric
   disc sizes can be compared at the same moment.

   It simulates a geocentric solar eclipse by overlapping
   the current geocentric sun moon discs to scale.

   Dark Disk   = Moon
   Yellow Disk = Sun
*/

   $WhichIsBiggerStr = ($AngDiamSun > $AngDiamMoon)? 'The solar disk is larger than the lunar disk with an':'The lunar disk is larger than the solar disk with an';
   if ($AngDiamDiffDeg < 0.0001666)  {$WhichIsBiggerStr = 'The sun and moon appear to be the same size.';}

   print <<< _HTML

<!DOCTYPE HTML>
<HTML
<head>
<title>Moon/Sun Apparent Sizes</title>
<meta name="viewport" content="width=device-width, initial-scale=1">
<meta http-equiv="content-type" content="text/html; charset=UTF-8">
<meta name="description" content="This PHP program computes an overlay comparing the current sizes of the geocentric Sun and Moon.">
<meta name="keywords" content="PHPScienceLabs.com">
<meta name="author" content="Jay Tanner - PHPScienceLabs.com">
<meta http-equiv="pragma"  content="no-cache">
<meta http-equiv="expires" content="-1">
<meta name="robots"    content="index,follow">
<meta name="googlebot" content="index,follow">
<link rel="canonical" href="https://www.phpsciencelabs.com/lunar-solar-current-disk-size-comparison/">

<style type='text/css'>
::selection      {background:yellow; color:black;}  /* For IE9+ and Safari     */
::-moz-selection {background:yellow; color:black;}  /* For FireFox and Mozilla */
</style>

</head>

<body>

<div ID='SmoothScroll' style='color:white; background-color:black; padding:6px; text-align:left;'>
<table width='800'><tr><td>
$PageHeaderText
<br><br>
<img src="Current_Moon_Sun_Size_Comparison.php" style='border-radius:8px;'>
<br>
<textarea cols='67' rows='29' ReadOnly style='color:white; background:#002200; padding:6px;
            font-family:monospace; font-size:150%; font-weight:bold; border:2px solid lime; border-radius:8px;' OnDblClick='this.select();' OnMouseUp='return true;'>
GEOCENTRIC MOON/SUN SIZE COMPARISONS FOR CURRENT DATE/TIME UT&nbsp;
NASA/JPL DE405 EPHEMERIS MODEL

Date      = $Y $MmmStr $dn $DoWStr       JD Number = $JDNumLT
Time      = $hhmmss UT

Delta T   = $dThhmmss
$_HR_[56]
JD TT     = $JDTT = Working Julian Date TT

MOON:
Ecl Lon   = $Lm&deg;
Ecl Lat   = $Bm&deg;
Distance  = $Distkm km = $Distmi mi
Radius    = 1738.05 km
Ang Diam  = $AngDiamMoon&deg;
Phase Ang = $PhaseDeg&deg;

SUN:
Ecl Lon   = $Ls&deg;
Ecl Lat   = $Bs&deg;
Distance  = $DistAU AU  =  $DistKm km  =  $DistMi mi
Radius    = 696,000 km
Ang Diam  = $AngDiamSun&deg;

$WhichIsBiggerStr
angular diameter difference of $AngDiamDiffDMS = $AngDiamDiffDeg&deg;
</textarea>

<br><br><br><br><br><br><br><br><br><br><br><br><br><br><br><br>
</td></tr></table>
</div>

<script src='../js/Smooth_Scroll.js'></script>
<script>SmoothScroll.init({cancelOnTouch: true});</script>

</body>

</HTML>


_HTML;




?>



